#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Windows IIS log parser."""

import unittest

from plaso.formatters import iis  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import iis

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


__author__ = 'Ashley Holtz (ashley.a.holtz@gmail.com)'


class WinIISUnitTest(test_lib.ParserTestCase):
  """Tests for the Windows IIS parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'iis.log'])
  def testParse(self):
    """Tests the Parse function."""
    parser = iis.WinIISParser()
    storage_writer = self._ParseFile([u'iis.log'], parser)

    self.assertEqual(storage_writer.number_of_events, 11)

    events = list(storage_writer.GetEvents())

    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-07-30 00:00:00')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.source_ip, u'10.10.10.100')
    self.assertEqual(event.dest_ip, u'10.10.10.100')
    self.assertEqual(event.dest_port, 80)

    expected_message = (
        u'GET /some/image/path/something.jpg '
        u'[ 10.10.10.100 > 10.10.10.100 : 80 ] '
        u'HTTP Status: 200 '
        u'User Agent: Mozilla/4.0+(compatible;+Win32;'
        u'+WinHttp.WinHttpRequest.5)')
    expected_short_message = (
        u'GET /some/image/path/something.jpg '
        u'[ 10.10.10.100 > 10.10.10.100 : 80 ]')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[5]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-07-30 00:00:05')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.http_method, u'GET')
    self.assertEqual(event.http_status, 200)
    self.assertEqual(
        event.requested_uri_stem, u'/some/image/path/something.jpg')

    event = events[1]

    expected_message = (
        u'GET /some/image/path/something.htm '
        u'[ 22.22.22.200 > 10.10.10.100 : 80 ] '
        u'HTTP Status: 404 '
        u'User Agent: Mozilla/5.0+(Macintosh;+Intel+Mac+OS+X+10_6_8)'
        u'+AppleWebKit/534.57.2+(KHTML,+like+Gecko)+Version/5.1.7'
        u'+Safari/534.57.2')
    expected_short_message = (
        u'GET /some/image/path/something.htm '
        u'[ 22.22.22.200 > 10.10.10.100 : 80 ]')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

  @shared_test_lib.skipUnlessHasTestFile([u'iis_without_date.log'])
  def testParseWithoutDate(self):
    """Tests the Parse function with logs without a date column."""
    parser = iis.WinIISParser()
    storage_writer = self._ParseFile([u'iis_without_date.log'], parser)

    self.assertEqual(storage_writer.number_of_events, 11)

    events = list(storage_writer.GetEvents())

    event = events[1]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-07-30 00:00:03')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.protocol_version, u'HTTP/1.1')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for Android WebView databases."""

from dfdatetime import java_time as dfdatetime_java_time
from dfdatetime import semantic_time as dfdatetime_semantic_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
# Register the cookie plugins.
from plaso.parsers import cookie_plugins  # pylint: disable=unused-import
from plaso.parsers import sqlite
from plaso.parsers.cookie_plugins import manager as cookie_plugins_manager
from plaso.parsers.sqlite_plugins import interface


class WebViewCookieEventData(events.EventData):
  """Android WebView cookie event data.

  Attributes:
    cookie_name (str): name of the cookie.
    data (str): data stored in the cookie.
    domain (str): host that set the cookie.
    path (str): path for which the cookie was set.
    secure (bool): True if the cookie should only be transmitted over
        a secure channel.
    url (str): URL of the cookie.
  """

  DATA_TYPE = u'webview:cookie'

  def __init__(self):
    """Initializes event data."""
    super(WebViewCookieEventData, self).__init__(data_type=self.DATA_TYPE)
    self.cookie_name = None
    self.data = None
    self.host = None
    self.path = None
    self.secure = None
    self.url = None


class WebViewPlugin(interface.SQLitePlugin):
  """Parser for WebView databases."""

  NAME = u'android_webview'
  DESCRIPTION = u'Parser for Android WebView databases'

  REQUIRED_TABLES = frozenset([u'android_metadata', u'cookies'])

  QUERIES = frozenset([
      (u'SELECT _id, name, value, domain, expires, path, secure FROM cookies',
       u'ParseCookieRow')])

  SCHEMAS = [{
      u'android_metadata': (
          u'CREATE TABLE android_metadata (locale TEXT)'),
      u'cookies': (
          u'CREATE TABLE cookies (_id INTEGER PRIMARY KEY, name TEXT, value '
          u'TEXT, domain TEXT, path TEXT, expires INTEGER, secure INTEGER)'),
      u'formdata': (
          u'CREATE TABLE formdata (_id INTEGER PRIMARY KEY, urlid INTEGER, '
          u'name TEXT, value TEXT, UNIQUE (urlid, name, value) ON CONFLICT '
          u'IGNORE)'),
      u'formurl': (
          u'CREATE TABLE formurl (_id INTEGER PRIMARY KEY, url TEXT)'),
      u'httpauth': (
          u'CREATE TABLE httpauth (_id INTEGER PRIMARY KEY, host TEXT, realm '
          u'TEXT, username TEXT, password TEXT, UNIQUE (host, realm) ON '
          u'CONFLICT REPLACE)'),
      u'password': (
          u'CREATE TABLE password (_id INTEGER PRIMARY KEY, host TEXT, '
          u'username TEXT, password TEXT, UNIQUE (host, username) ON CONFLICT '
          u'REPLACE)')}]

  def __init__(self):
    """Initializes a plugin object."""
    super(WebViewPlugin, self).__init__()
    self._cookie_plugins = (
        cookie_plugins_manager.CookiePluginsManager.GetPlugins())

  def ParseCookieRow(self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses a row from the database.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string']
    # and will raise "IndexError: Index must be int or string". All indexes are
    # thus raw strings.

    cookie_name = row['name']
    cookie_value = row['value']
    path = row['path']

    hostname = row['domain']
    if hostname.startswith('.'):
      hostname = hostname[1:]

    # The WebView database stores the secure flag as a integer type,
    # but we represent it as a boolean.
    secure = row['secure'] != 0

    if secure:
      scheme = u'https'
    else:
      scheme = u'http'

    url = u'{0:s}://{1:s}{2:s}'.format(scheme, hostname, path)

    timestamp = row['expires']
    if timestamp:
      date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
    else:
      date_time = dfdatetime_semantic_time.SemanticTime(u'Infinity')

    event_data = WebViewCookieEventData()
    event_data.cookie_name = cookie_name
    event_data.data = cookie_value
    event_data.host = hostname
    event_data.offset = row['_id']
    event_data.path = path
    event_data.query = query
    event_data.secure = secure
    event_data.url = url

    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_EXPIRATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    # Go through all cookie plugins to see if there are is any specific parsing
    # needed.
    for cookie_plugin in self._cookie_plugins:
      try:
        cookie_plugin.UpdateChainAndProcess(
            parser_mediator, cookie_name=cookie_name,
            cookie_data=cookie_value, url=url)
      except errors.WrongPlugin:
        pass


sqlite.SQLiteParser.RegisterPlugin(WebViewPlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Popularity Contest event formatters."""

import unittest

from plaso.formatters import popcontest

from tests.formatters import test_lib


class PopularityContestSessionFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the PCAP event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = popcontest.PopularityContestSessionFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = popcontest.PopularityContestSessionFormatter()

    expected_attribute_names = [
        u'session',
        u'status',
        u'hostid',
        u'details']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class PopularityContestLogFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the PCAP event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = popcontest.PopularityContestLogFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = popcontest.PopularityContestLogFormatter()

    expected_attribute_names = [
        u'mru',
        u'package',
        u'record_tag']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Airport plist plugin."""

from dfdatetime import posix_time as dfdatetime_posix_time
from dfdatetime import semantic_time as dfdatetime_semantic_time

from plaso.containers import plist_event
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


class AirportPlugin(interface.PlistPlugin):
  """Plist plugin that extracts WiFi information."""

  NAME = u'airport'
  DESCRIPTION = u'Parser for Airport plist files.'

  PLIST_PATH = u'com.apple.airport.preferences.plist'
  PLIST_KEYS = frozenset([u'RememberedNetworks'])

  def GetEntries(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant Airport entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    if u'RememberedNetworks' not in match:
      return

    for wifi in match[u'RememberedNetworks']:
      ssid = wifi.get(u'SSIDString', u'UNKNOWN_SSID')
      security_type = wifi.get(u'SecurityType', u'UNKNOWN_SECURITY_TYPE')

      event_data = plist_event.PlistTimeEventData()
      event_data.desc = (
          u'[WiFi] Connected to network: <{0:s}> using security {1:s}').format(
              ssid, security_type)
      event_data.key = u'item'
      event_data.root = u'/RememberedNetworks'

      datetime_value = wifi.get(u'LastConnected', None)
      if datetime_value:
        timestamp = timelib.Timestamp.FromPythonDatetime(datetime_value)
        date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
            timestamp=timestamp)
        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_WRITTEN)
      else:
        date_time = dfdatetime_semantic_time.SemanticTime(u'Not set')
        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_NOT_A_TIME)

      parser_mediator.ProduceEventWithEventData(event, event_data)


plist.PlistParser.RegisterPlugin(AirportPlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the SAM Users Account information plugin."""

import unittest

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers.winreg_plugins import sam_users

from tests import test_lib as shared_test_lib
from tests.parsers.winreg_plugins import test_lib


__author__ = 'Preston Miller, dpmforensics.com, github.com/prmiller91'


class SAMUsersWindowsRegistryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests the SAM Users Account information plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'SAM'])
  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry([u'SAM'])
    key_path = u'HKEY_LOCAL_MACHINE\\SAM\\SAM\\Domains\\Account\\Users'

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = sam_users.SAMUsersWindowsRegistryPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_events, 7)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self._TestRegvalue(event, u'account_rid', 500)
    self._TestRegvalue(event, u'login_count', 6)
    self._TestRegvalue(event, u'username', u'Administrator')

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-09-24 03:36:06.358837')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_WRITTEN)

    expected_message = (
        u'[{0:s}] '
        u'account_rid: 500 '
        u'comments: Built-in account for administering the computer/domain '
        u'login_count: 6 '
        u'username: Administrator').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    # Test SAMUsersWindowsRegistryEvent.
    event = events[1]

    self.assertEqual(event.account_rid, 500)
    self.assertEqual(event.login_count, 6)
    self.assertEqual(event.username, u'Administrator')

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-11-20 21:48:12.569244')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_LAST_LOGIN)

    expected_message = (
        u'[{0:s}] '
        u'Username: Administrator '
        u'Comments: Built-in account for administering the computer/domain '
        u'RID: 500 '
        u'Login count: 6').format(key_path)
    expected_short_message = (
        u'Administrator '
        u'RID: 500 '
        u'Login count: 6')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the NetworkList registry plugin."""

import binascii

import construct

from dfdatetime import systemtime as dfdatetime_systemtime

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import winreg
from plaso.parsers.winreg_plugins import interface


class WindowsRegistryNetworkEventData(events.EventData):
  """Windows network event data.

  Attributes:
    connection_type (str): type of connection.
    default_gateway_mac (str): MAC address for the default gateway.
    description (str): description of the wireless connection.
    dns_suffix (str): DNS suffix.
    ssid (str): SSID of the connection.
  """

  DATA_TYPE = u'windows:registry:network'

  def __init__(self):
    """Initializes event data."""
    super(WindowsRegistryNetworkEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.connection_type = None
    self.default_gateway_mac = None
    self.description = None
    self.dns_suffix = None
    self.ssid = None


class NetworksPlugin(interface.WindowsRegistryPlugin):
  """Windows Registry plugin for parsing the NetworkList key."""

  NAME = u'networks'
  DESCRIPTION = u'Parser for NetworkList data.'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          u'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion'
          u'\\NetworkList')])

  _CONNECTION_TYPE = {
      0x06: u'Wired',
      0x17: u'3g',
      0x47: u'Wireless'}

  _EMPTY_SYSTEM_TIME_TUPLE = (0, 0, 0, 0, 0, 0, 0, 0)

  _SYSTEMTIME_STRUCT = construct.Struct(
      u'systemtime',
      construct.ULInt16(u'year'),
      construct.ULInt16(u'month'),
      construct.ULInt16(u'day_of_week'),
      construct.ULInt16(u'day_of_month'),
      construct.ULInt16(u'hours'),
      construct.ULInt16(u'minutes'),
      construct.ULInt16(u'seconds'),
      construct.ULInt16(u'milliseconds'))

  def _GetNetworkInfo(self, signatures_key):
    """Retrieves the network info within the signatures subkey.

    Args:
      signatures_key (dfwinreg.WinRegistryKey): a Windows Registry key.

    Returns:
      A dictionary containing tuples (default_gateway_mac, dns_suffix) hashed
      by profile guid.
    """
    network_info = {}
    for category in signatures_key.GetSubkeys():
      for signature in category.GetSubkeys():
        profile_guid_value = signature.GetValueByName(u'ProfileGuid')
        if profile_guid_value:
          profile_guid = profile_guid_value.GetDataAsObject()
        else:
          continue

        default_gateway_mac_value = signature.GetValueByName(
            u'DefaultGatewayMac')
        if default_gateway_mac_value:
          default_gateway_mac = default_gateway_mac_value.GetDataAsObject()
          default_gateway_mac = u':'.join(
              map(binascii.hexlify, default_gateway_mac))
        else:
          default_gateway_mac = None

        dns_suffix_value = signature.GetValueByName(u'DnsSuffix')
        if dns_suffix_value:
          dns_suffix = dns_suffix_value.GetDataAsObject()
        else:
          dns_suffix = None

        network_info[profile_guid] = (default_gateway_mac, dns_suffix)

    return network_info

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    network_info = {}
    signatures = registry_key.GetSubkeyByName(u'Signatures')
    if signatures:
      network_info = self._GetNetworkInfo(signatures)

    profiles = registry_key.GetSubkeyByName(u'Profiles')
    if not profiles:
      return

    for subkey in profiles.GetSubkeys():
      default_gateway_mac, dns_suffix = network_info.get(
          subkey.name, (None, None))

      event_data = WindowsRegistryNetworkEventData()
      event_data.default_gateway_mac = default_gateway_mac
      event_data.dns_suffix = dns_suffix

      ssid_value = subkey.GetValueByName(u'ProfileName')
      if ssid_value:
        event_data.ssid = ssid_value.GetDataAsObject()

      description_value = subkey.GetValueByName(u'Description')
      if description_value:
        event_data.description = description_value.GetDataAsObject()

      connection_type_value = subkey.GetValueByName(u'NameType')
      if connection_type_value:
        connection_type = connection_type_value.GetDataAsObject()
        # TODO: move to formatter.
        connection_type = self._CONNECTION_TYPE.get(
            connection_type, u'unknown')
        event_data.connection_type = connection_type

      date_created_value = subkey.GetValueByName(u'DateCreated')
      if date_created_value:
        try:
          systemtime_struct = self._SYSTEMTIME_STRUCT.parse(
              date_created_value.data)
        except construct.ConstructError as exception:
          systemtime_struct = None
          parser_mediator.ProduceExtractionError(
              u'unable to parse date created with error: {0:s}'.format(
                  exception))

        system_time_tuple = self._EMPTY_SYSTEM_TIME_TUPLE
        if systemtime_struct:
          system_time_tuple = (
              systemtime_struct.year, systemtime_struct.month,
              systemtime_struct.day_of_week, systemtime_struct.day_of_month,
              systemtime_struct.hours, systemtime_struct.minutes,
              systemtime_struct.seconds, systemtime_struct.milliseconds)

        date_time = None
        if system_time_tuple != self._EMPTY_SYSTEM_TIME_TUPLE:
          try:
            date_time = dfdatetime_systemtime.Systemtime(
                system_time_tuple=system_time_tuple)
          except ValueError:
            parser_mediator.ProduceExtractionError(
                u'invalid system time: {0!s}'.format(system_time_tuple))

        if date_time:
          event = time_events.DateTimeValuesEvent(
              date_time, definitions.TIME_DESCRIPTION_CREATION)
          parser_mediator.ProduceEventWithEventData(event, event_data)

      date_last_connected_value = subkey.GetValueByName(u'DateLastConnected')
      if date_last_connected_value:
        try:
          systemtime_struct = self._SYSTEMTIME_STRUCT.parse(
              date_last_connected_value.data)
        except construct.ConstructError as exception:
          systemtime_struct = None
          parser_mediator.ProduceExtractionError(
              u'unable to parse date last connected with error: {0:s}'.format(
                  exception))

        system_time_tuple = self._EMPTY_SYSTEM_TIME_TUPLE
        if systemtime_struct:
          system_time_tuple = (
              systemtime_struct.year, systemtime_struct.month,
              systemtime_struct.day_of_week, systemtime_struct.day_of_month,
              systemtime_struct.hours, systemtime_struct.minutes,
              systemtime_struct.seconds, systemtime_struct.milliseconds)

        date_time = None
        if system_time_tuple != self._EMPTY_SYSTEM_TIME_TUPLE:
          try:
            date_time = dfdatetime_systemtime.Systemtime(
                system_time_tuple=system_time_tuple)
          except ValueError:
            parser_mediator.ProduceExtractionError(
                u'invalid system time: {0!s}'.format(system_time_tuple))

        if date_time:
          event = time_events.DateTimeValuesEvent(
              date_time, definitions.TIME_DESCRIPTION_LAST_CONNECTED)
          parser_mediator.ProduceEventWithEventData(event, event_data)


winreg.WinRegistryParser.RegisterPlugin(NetworksPlugin)



# -*- coding: utf-8 -*-
"""The analysis plugins CLI arguments helper."""

import sys

from plaso.analysis import manager as analysis_manager
from plaso.cli import tools
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors


class AnalysisPluginsArgumentsHelper(interface.ArgumentsHelper):
  """Analysis plugins CLI arguments helper."""

  NAME = u'analysis_plugins'
  DESCRIPTION = u'Analysis plugins command line arguments.'

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        u'--analysis', metavar=u'PLUGIN_LIST', dest=u'analysis_plugins',
        default=u'', action=u'store', type=str, help=(
            u'A comma separated list of analysis plugin names to be loaded '
            u'or "--analysis list" to see a list of available plugins.'))

    arguments = sys.argv[1:]
    argument_index = 0

    if u'--analysis' in arguments:
      argument_index = arguments.index(u'--analysis') + 1

    if argument_index > 0 and argument_index < len(arguments):
      names = [name.strip() for name in arguments[argument_index].split(u',')]
    else:
      names = None

    if names and names != [u'list']:
      manager.ArgumentHelperManager.AddCommandLineArguments(
          argument_group, category=u'analysis', names=names)

  @classmethod
  def ParseOptions(cls, options, configuration_object):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      configuration_object (CLITool): object to be configured by the argument
          helper.

    Raises:
      BadConfigObject: when the configuration object is of the wrong type.
    """
    if not isinstance(configuration_object, tools.CLITool):
      raise errors.BadConfigObject(
          u'Configuration object is not an instance of CLITool')

    plugin_names_argument = cls._ParseStringOption(options, u'analysis_plugins')

    if plugin_names_argument and plugin_names_argument != u'list':
      available_plugin_names = (
          analysis_manager.AnalysisPluginManager.GetPluginNames())
      plugin_names_argument = [
          name.strip() for name in plugin_names_argument.split(u',')]

      difference = set(plugin_names_argument).difference(available_plugin_names)
      if difference:
        raise errors.BadConfigOption(
            u'Non-existent analysis plugins specified: {0:s}'.format(
                u' '.join(difference)))

    setattr(configuration_object, u'_analysis_plugins', plugin_names_argument)


manager.ArgumentHelperManager.RegisterHelper(AnalysisPluginsArgumentsHelper)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the 4n6time MySQL output class."""

import unittest

from plaso.containers import events
from plaso import formatters   # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.output import mysql_4n6time

from tests.output import fake_mysqldb
from tests.output import test_lib

if mysql_4n6time.MySQLdb is None:
  mysql_4n6time.MySQLdb = fake_mysqldb


class MySQL4n6TimeTestEvent(events.EventObject):
  """Test event."""

  DATA_TYPE = u'syslog:line'

  def __init__(self, event_timestamp):
    """Initializes an event."""
    super(MySQL4n6TimeTestEvent, self).__init__()
    self.display_name = u'log/syslog.1'
    self.filename = u'log/syslog.1'
    self.hostname = u'ubuntu'
    self.my_number = 123
    self.some_additional_foo = True
    self.text = (
        u'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session '
        u'closed for user root)')
    self.timestamp_desc = definitions.TIME_DESCRIPTION_WRITTEN
    self.timestamp = event_timestamp


class MySQL4n6TimeOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests for the 4n6time MySQL output class."""

  # pylint: disable=protected-access

  def testGetTags(self):
    """Tests the _GetTags function."""
    fake_cursor = fake_mysqldb.FakeMySQLdbCursor()
    fake_cursor.expected_query = u'SELECT DISTINCT tag FROM log2timeline'
    fake_cursor.query_results = [(u'one',), (u'two,three',), (u'four',)]

    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)
    output_module._cursor = fake_cursor

    expected_tags = [u'one', u'two', u'three', u'four']
    tags = output_module._GetTags()
    self.assertEqual(tags, expected_tags)

  def testGetUniqueValues(self):
    """Tests the _GetUniqueValues function."""
    fake_cursor = fake_mysqldb.FakeMySQLdbCursor()
    fake_cursor.expected_query = (
        u'SELECT source, COUNT(source) FROM log2timeline GROUP BY source')
    fake_cursor.query_results = [(u'one', 1), (u'two', 2), (u'three', 3)]

    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)
    output_module._cursor = fake_cursor

    expected_unique_values = {u'one': 1, u'two': 2, u'three': 3}
    unique_values = output_module._GetUniqueValues(u'source')
    self.assertEqual(unique_values, expected_unique_values)

  # TODO: add test for Open and Close

  def testGetSanitizedEventValues(self):
    """Tests the GetSanitizedEventValues function."""
    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)

    expected_dict = {
        u'type': u'Content Modification Time',
        u'host': u'ubuntu',
        u'filename': u'log/syslog.1',
        u'source': u'LOG',
        u'description': u'[',
        u'datetime': u'2012-06-27 18:17:01',
        u'inreport': u'',
        u'source_name': u'-',
        u'extra': (
            u'my_number: 123  some_additional_foo: True  text: '
            u'Reporter <CRON> PID: 8442 (pam_unix(cron:session): '
            u'session closed for user root) '
        ),
        u'tag': u'',
        u'timezone': u'UTC',
        u'inode': u'-',
        u'reportnotes': u'',
        u'sourcetype': u'Log File',
        u'event_identifier': u'-',
        u'format': u'-',
        u'URL': u'-',
        u'record_number': 0,
        u'MACB': u'M...',
        u'computer_name': u'-',
        u'offset': 0,
        u'evidence': u'-',
        u'user_sid': u'-',
        u'notes': u'-',
        u'vss_store_number': -1,
        u'user': u'-'
    }

    timestamp = timelib.Timestamp.CopyFromString(
        u'2012-06-27 18:17:01+00:00')
    event = MySQL4n6TimeTestEvent(timestamp)
    event_dict = output_module._GetSanitizedEventValues(event)

    self.assertIsInstance(event_dict, dict)
    self.assertDictContainsSubset(expected_dict, event_dict)

  def testSetCredentials(self):
    """Tests the SetCredentials function."""
    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)

    output_module.SetCredentials(password=u'password', username=u'username')
    self.assertEqual(output_module._password, u'password')
    self.assertEqual(output_module._user, u'username')

  def testSetDatabaseName(self):
    """Tests the SetDatabaseName function."""
    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)

    output_module.SetDatabaseName(u'database')
    self.assertEqual(output_module._dbname, u'database')

  def testSetServerInformation(self):
    """Tests the SetServerInformation function."""
    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)

    output_module.SetServerInformation(u'127.0.0.1', 3306)
    self.assertEqual(output_module._host, u'127.0.0.1')
    self.assertEqual(output_module._port, 3306)

  def testWriteEventBody(self):
    """Tests the WriteEventBody function."""
    fake_cursor = fake_mysqldb.FakeMySQLdbCursor()
    fake_cursor.expected_query = (
        mysql_4n6time.MySQL4n6TimeOutputModule._INSERT_QUERY)

    fake_cursor.expected_query_args = {
        u'computer_name': u'-',
        u'datetime': u'2012-06-27 18:17:01',
        u'description': u'[',
        u'event_identifier': u'-',
        u'event_type': u'-',
        u'evidence': u'-',
        u'extra': (
            u'my_number: 123  '
            u'some_additional_foo: True  '
            u'text: Reporter <CRON> PID: 8442 (pam_unix(cron:session): '
            u'session closed for user root) '),
        u'filename': u'log/syslog.1',
        u'format': u'-',
        u'host': u'ubuntu',
        u'inode': u'-',
        u'inreport': u'',
        u'MACB': u'M...',
        u'notes': u'-',
        u'offset': 0,
        u'record_number': 0,
        u'reportnotes': u'',
        u'source_name': u'-',
        u'sourcetype': u'Log File',
        u'source': u'LOG',
        u'tag': u'',
        u'timezone': u'UTC',
        u'type': u'Content Modification Time',
        u'URL': u'-',
        u'user_sid': u'-',
        u'user': u'-',
        u'vss_store_number': -1}

    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)
    output_module._count = 0
    output_module._cursor = fake_cursor

    timestamp = timelib.Timestamp.CopyFromString(
        u'2012-06-27 18:17:01+00:00')
    event = MySQL4n6TimeTestEvent(timestamp)
    output_module.WriteEventBody(event)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains a parser for the Google Drive snapshots.

The Google Drive snapshots are stored in SQLite database files named
snapshot.db.
"""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


__author__ = 'David Nides (david.nides@gmail.com)'


class GoogleDriveSnapshotCloudEntryEventData(events.EventData):
  """Google Drive snapshot cloud entry event data.

  Attributes:
    doc_type (int): document type.
    path (str): path of the file.
    shared (bool): True if the file is shared, False if the file is private.
    size (int): size of the file.
    url (str): URL of the file.
  """

  DATA_TYPE = u'gdrive:snapshot:cloud_entry'

  def __init__(self):
    """Initializes event data."""
    super(GoogleDriveSnapshotCloudEntryEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.document_type = None
    self.path = None
    self.shared = None
    self.size = None
    self.url = None


class GoogleDriveSnapshotLocalEntryEventData(events.EventData):
  """Google Drive snapshot local entry event data.

  Attributes:
    path (str): path of the file.
    size (int): size of the file.
  """

  DATA_TYPE = u'gdrive:snapshot:local_entry'

  def __init__(self):
    """Initializes event data."""
    super(GoogleDriveSnapshotLocalEntryEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.path = None
    self.size = None


class GoogleDrivePlugin(interface.SQLitePlugin):
  """SQLite plugin for Google Drive snapshot.db files."""

  NAME = u'google_drive'
  DESCRIPTION = u'Parser for Google Drive SQLite database files.'

  # Define the needed queries.
  QUERIES = [
      ((u'SELECT cloud_entry.resource_id, cloud_entry.filename, '
        u'cloud_entry.modified, cloud_entry.created, cloud_entry.size, '
        u'cloud_entry.doc_type, cloud_entry.shared, cloud_entry.checksum, '
        u'cloud_entry.url, cloud_relations.parent_resource_id '
        u'FROM cloud_entry, cloud_relations '
        u'WHERE cloud_relations.child_resource_id = cloud_entry.resource_id '
        u'AND cloud_entry.modified IS NOT NULL;'),
       u'ParseCloudEntryRow'),
      ((u'SELECT inode_number, filename, modified, checksum, size '
        u'FROM local_entry WHERE modified IS NOT NULL;'),
       u'ParseLocalEntryRow')]

  # The required tables.
  REQUIRED_TABLES = frozenset([
      u'cloud_entry', u'cloud_relations', u'local_entry', u'local_relations',
      u'mapping', u'overlay_status'])

  SCHEMAS = [{
      u'cloud_entry': (
          u'CREATE TABLE cloud_entry (resource_id TEXT, filename TEXT, '
          u'modified INTEGER, created INTEGER, acl_role INTEGER, doc_type '
          u'INTEGER, removed INTEGER, url TEXT, size INTEGER, checksum TEXT, '
          u'shared INTEGER, PRIMARY KEY (resource_id))'),
      u'cloud_relations': (
          u'CREATE TABLE cloud_relations (child_resource_id TEXT, '
          u'parent_resource_id TEXT, UNIQUE (child_resource_id, '
          u'parent_resource_id), FOREIGN KEY (child_resource_id) REFERENCES '
          u'cloud_entry(resource_id), FOREIGN KEY (parent_resource_id) '
          u'REFERENCES cloud_entry(resource_id))'),
      u'local_entry': (
          u'CREATE TABLE local_entry (inode_number INTEGER, filename TEXT, '
          u'modified INTEGER, checksum TEXT, size INTEGER, PRIMARY KEY '
          u'(inode_number))'),
      u'local_relations': (
          u'CREATE TABLE local_relations (child_inode_number INTEGER, '
          u'parent_inode_number INTEGER, UNIQUE (child_inode_number), FOREIGN '
          u'KEY (parent_inode_number) REFERENCES local_entry(inode_number), '
          u'FOREIGN KEY (child_inode_number) REFERENCES '
          u'local_entry(inode_number))'),
      u'mapping': (
          u'CREATE TABLE mapping (inode_number INTEGER, resource_id TEXT, '
          u'UNIQUE (inode_number), FOREIGN KEY (inode_number) REFERENCES '
          u'local_entry(inode_number), FOREIGN KEY (resource_id) REFERENCES '
          u'cloud_entry(resource_id))'),
      u'overlay_status': (
          u'CREATE TABLE overlay_status (path TEXT, overlay_status INTEGER, '
          u'PRIMARY KEY (path))')}]

  # Queries used to build cache.
  LOCAL_PATH_CACHE_QUERY = (
      u'SELECT local_relations.child_inode_number, '
      u'local_relations.parent_inode_number, local_entry.filename '
      u'FROM local_relations, local_entry '
      u'WHERE local_relations.child_inode_number = local_entry.inode_number')
  CLOUD_PATH_CACHE_QUERY = (
      u'SELECT cloud_entry.filename, cloud_entry.resource_id, '
      u'cloud_relations.parent_resource_id AS parent '
      u'FROM cloud_entry, cloud_relations '
      u'WHERE cloud_entry.doc_type = 0 '
      u'AND cloud_entry.resource_id = cloud_relations.child_resource_id')

  def GetLocalPath(self, inode, cache, database):
    """Return local path for a given inode.

    Args:
      inode: The inode number for the file.
      cache (SQLiteCache): cache.
      database (SQLiteDatabase): database.

    Returns:
      A full path, including the filename of the given inode value.
    """
    local_path = cache.GetResults(u'local_path')
    if not local_path:
      results = database.Query(self.LOCAL_PATH_CACHE_QUERY)

      # Note that pysqlite does not accept a Unicode string in row['string'] and
      # will raise "IndexError: Index must be int or string".
      cache.CacheQueryResults(
          results, 'local_path', 'child_inode_number',
          ('parent_inode_number', 'filename'))
      local_path = cache.GetResults(u'local_path')

    parent, path = local_path.get(inode, [None, None])

    # TODO: Read the local_sync_root from the sync_config.db and use that
    # for a root value.
    root_value = u'%local_sync_root%/'

    if not path:
      return root_value

    paths = []
    while path:
      paths.append(path)
      parent, path = local_path.get(parent, [None, None])

    if not paths:
      return root_value

    # Paths are built top level to root so we need to reverse the list to
    # represent them in the traditional order.
    paths.reverse()
    return root_value + u'/'.join(paths)

  def GetCloudPath(self, resource_id, cache, database):
    """Return cloud path given a resource id.

    Args:
      resource_id: The resource_id for the file.
      cache: The local cache object.
      database: A database object (instance of SQLiteDatabase).

    Returns:
      A full path to the resource value.
    """
    cloud_path = cache.GetResults(u'cloud_path')
    if not cloud_path:
      results = database.Query(self.CLOUD_PATH_CACHE_QUERY)

      # Note that pysqlite does not accept a Unicode string in row['string'] and
      # will raise "IndexError: Index must be int or string".
      cache.CacheQueryResults(
          results, 'cloud_path', 'resource_id', ('filename', 'parent'))
      cloud_path = cache.GetResults(u'cloud_path')

    if resource_id == u'folder:root':
      return u'/'

    paths = []
    parent_path, parent_id = cloud_path.get(resource_id, [u'', u''])
    while parent_path:
      if parent_path == u'folder:root':
        break
      paths.append(parent_path)
      parent_path, parent_id = cloud_path.get(parent_id, [u'', u''])

    if not paths:
      return u'/'

    # Paths are built top level to root so we need to reverse the list to
    # represent them in the traditional order.
    paths.reverse()
    return u'/{0:s}/'.format(u'/'.join(paths))

  def ParseCloudEntryRow(
      self, parser_mediator, row, cache=None, database=None, query=None,
      **unused_kwargs):
    """Parses a cloud entry row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      cache (SQLiteCache): cache.
      database (SQLiteDatabase): database.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    cloud_path = self.GetCloudPath(row['parent_resource_id'], cache, database)
    cloud_filename = u'{0:s}{1:s}'.format(cloud_path, row['filename'])

    event_data = GoogleDriveSnapshotCloudEntryEventData()
    event_data.document_type = row['doc_type']
    event_data.path = cloud_filename
    event_data.query = query
    event_data.shared = bool(row['shared'])
    event_data.size = row['size']
    event_data.url = row['url']

    date_time = dfdatetime_posix_time.PosixTime(timestamp=row['modified'])
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    if row['created']:
      date_time = dfdatetime_posix_time.PosixTime(timestamp=row['created'])
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

  def ParseLocalEntryRow(
      self, parser_mediator, row, cache=None, database=None, query=None,
      **unused_kwargs):
    """Parses a local entry row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      cache (SQLiteCache): cache.
      database (SQLiteDatabase): database.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    local_path = self.GetLocalPath(row['inode_number'], cache, database)

    event_data = GoogleDriveSnapshotLocalEntryEventData()
    event_data.path = local_path
    event_data.query = query
    event_data.size = row['size']

    date_time = dfdatetime_posix_time.PosixTime(timestamp=row['modified'])
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(GoogleDrivePlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the nsrlsvr analysis plugin CLI arguments helper."""

import argparse
import unittest

from plaso.analysis import nsrlsvr
from plaso.lib import errors
from plaso.cli.helpers import nsrlsvr_analysis

from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


class NsrlsvrAnalysisArgumentsHelperTest(
    test_lib.AnalysisPluginArgumentsHelperTest):
  """Tests the nsrlsvr analysis plugin CLI arguments helper."""

  _EXPECTED_OUTPUT = u'\n'.join([
      u'usage: cli_helper.py [--nsrlsvr-hash HASH] [--nsrlsvr-host HOST]',
      u'                     [--nsrlsvr-label LABEL] [--nsrlsvr-port PORT]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  --nsrlsvr-hash HASH, --nsrlsvr_hash HASH',
      (u'                        Type of hash to use to query nsrlsvr '
       u'instance, the'),
      (u'                        default is: md5. Supported options: md5, '
       u'sha1'),
      u'  --nsrlsvr-host HOST, --nsrlsvr_host HOST',
      (u'                        Hostname or IP address of the nsrlsvr '
       u'instance to'),
      u'                        query, the default is: localhost',
      u'  --nsrlsvr-label LABEL, --nsrlsvr_label LABEL',
      u'                        Label to apply to events, the default is:',
      u'                        nsrl_present.',
      u'  --nsrlsvr-port PORT, --nsrlsvr_port PORT',
      (u'                        Port number of the nsrlsvr instance to '
       u'query, the'),
      u'                        default is: 9120.',
      u''])

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'cli_helper.py',
        description=u'Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    nsrlsvr_analysis.NsrlsvrAnalysisArgumentsHelper.AddArguments(
        argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    analysis_plugin = nsrlsvr.NsrlsvrAnalysisPlugin()

    with self.assertRaises(errors.BadConfigOption):
      nsrlsvr_analysis.NsrlsvrAnalysisArgumentsHelper.ParseOptions(
          options, analysis_plugin)

    with self.assertRaises(errors.BadConfigObject):
      nsrlsvr_analysis.NsrlsvrAnalysisArgumentsHelper.ParseOptions(
          options, None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the object filter functions."""

import unittest

from plaso.lib import errors
from plaso.lib import objectfilter


# pylint: disable=missing-docstring

class DummyObject(object):
  def __init__(self, key, value):
    setattr(self, key, value)


class HashObject(object):
  def __init__(self, hash_value=None):
    self.value = hash_value

  @property
  def md5(self):
    return self.value

  def __eq__(self, y):
    return self.value == y

  def __lt__(self, y):
    return self.value < y


class Dll(object):
  def __init__(self, name, imported_functions=None, exported_functions=None):
    self.name = name
    self._imported_functions = imported_functions or []
    self.num_imported_functions = len(self._imported_functions)
    self.exported_functions = exported_functions or []
    self.num_exported_functions = len(self.exported_functions)

  @property
  def imported_functions(self):
    for fn in self._imported_functions:
      yield fn


class DummyFile(object):
  _FILENAME = 'boot.ini'

  ATTR1 = 'Backup'
  ATTR2 = 'Archive'
  HASH1 = '123abc'
  HASH2 = '456def'

  non_callable_leaf = 'yoda'

  def __init__(self):
    self.non_callable = HashObject(self.HASH1)
    self.non_callable_repeated = [
        DummyObject('desmond', ['brotha', 'brotha']),
        DummyObject('desmond', ['brotha', 'sista'])]
    self.imported_dll1 = Dll('a.dll', ['FindWindow', 'CreateFileA'])
    self.imported_dll2 = Dll('b.dll', ['RegQueryValueEx'])

  @property
  def name(self):
    return self._FILENAME

  @property
  def attributes(self):
    return [self.ATTR1, self.ATTR2]

  @property
  def hash(self):
    return [HashObject(self.HASH1), HashObject(self.HASH2)]

  @property
  def size(self):
    return 10

  @property
  def deferred_values(self):
    for v in ['a', 'b']:
      yield v

  @property
  def novalues(self):
    return []

  @property
  def imported_dlls(self):
    return [self.imported_dll1, self.imported_dll2]

  def Callable(self):
    raise RuntimeError(u'This can not be called.')

  @property
  def float(self):
    return 123.9823


class ObjectFilterTest(unittest.TestCase):

  def setUp(self):
    """Makes preparations before running an individual test."""
    self.file = DummyFile()
    self.filter_imp = objectfilter.LowercaseAttributeFilterImplementation
    self.value_expander = self.filter_imp.FILTERS['ValueExpander']

  operator_tests = {
      objectfilter.Less: [
          (True, ['size', 1000]),
          (True, ['size', 11]),
          (False, ['size', 10]),
          (False, ['size', 0]),
          (False, ['float', 1.0]),
          (True, ['float', 123.9824])],
      objectfilter.LessEqual: [
          (True, ['size', 1000]),
          (True, ['size', 11]),
          (True, ['size', 10]),
          (False, ['size', 9]),
          (False, ['float', 1.0]),
          (True, ['float', 123.9823])],
      objectfilter.Greater: [
          (True, ['size', 1]),
          (True, ['size', 9.23]),
          (False, ['size', 10]),
          (False, ['size', 1000]),
          (True, ['float', 122]),
          (True, ['float', 1.0])],
      objectfilter.GreaterEqual: [
          (False, ['size', 1000]),
          (False, ['size', 11]),
          (True, ['size', 10]),
          (True, ['size', 0]),
          # Floats work fine too.
          (True, ['float', 122]),
          (True, ['float', 123.9823]),
          # Comparisons works with strings, although it might be a bit silly.
          (True, ['name', 'aoot.ini'])],
      objectfilter.Contains: [
          # Contains works with strings.
          (True, ['name', 'boot.ini']),
          (True, ['name', 'boot']),
          (False, ['name', 'meh']),
          # Works with generators.
          (True, ['imported_dlls.imported_functions', 'FindWindow']),
          # But not with numbers.
          (False, ['size', 12])],
      objectfilter.Equals: [
          (True, ['name', 'boot.ini']),
          (False, ['name', 'foobar']),
          (True, ['float', 123.9823])],
      objectfilter.NotEquals: [
          (False, ['name', 'boot.ini']),
          (True, ['name', 'foobar']),
          (True, ['float', 25])],
      objectfilter.InSet: [
          (True, ['name', ['boot.ini', 'autoexec.bat']]),
          (True, ['name', 'boot.ini']),
          (False, ['name', 'NOPE']),
          # All values of attributes are within these.
          (True, ['attributes', ['Archive', 'Backup', 'Nonexisting']]),
          # Not all values of attributes are within these.
          (False, ['attributes', ['Executable', 'Sparse']])],
      objectfilter.Regexp: [
          (True, ['name', '^boot.ini$']),
          (True, ['name', 'boot.ini']),
          (False, ['name', '^$']),
          (True, ['attributes', 'Archive']),
          # One can regexp numbers if he's inclined to.
          (True, ['size', 0]),
          # But regexp doesn't work with lists or generators for the moment.
          (False, ['imported_dlls.imported_functions', 'FindWindow'])],
      }

  def testBinaryOperators(self):
    for operator, test_data in self.operator_tests.items():
      for test_unit in test_data:
        kwargs = {'arguments': test_unit[1],
                  'value_expander': self.value_expander}
        ops = operator(**kwargs)
        self.assertEqual(test_unit[0], ops.Matches(self.file))
        if hasattr(ops, 'FlipBool'):
          ops.FlipBool()
          self.assertEqual(not test_unit[0], ops.Matches(self.file))

  def testExpand(self):
    # Case insensitivity.
    values_lowercase = self.value_expander().Expand(self.file, 'size')
    values_uppercase = self.value_expander().Expand(self.file, 'Size')
    self.assertListEqual(list(values_lowercase), list(values_uppercase))

    # Existing, non-repeated, leaf is a value.
    values = self.value_expander().Expand(self.file, 'size')
    self.assertListEqual(list(values), [10])

    # Existing, non-repeated, leaf is iterable.
    values = self.value_expander().Expand(self.file, 'attributes')
    self.assertListEqual(list(values), [[DummyFile.ATTR1, DummyFile.ATTR2]])

    # Existing, repeated, leaf is value.
    values = self.value_expander().Expand(self.file, 'hash.md5')
    self.assertListEqual(list(values), [DummyFile.HASH1, DummyFile.HASH2])

    # Existing, repeated, leaf is iterable.
    values = self.value_expander().Expand(
        self.file, 'non_callable_repeated.desmond')
    self.assertListEqual(
        list(values), [['brotha', 'brotha'], ['brotha', 'sista']])

    # Now with an iterator.
    values = self.value_expander().Expand(self.file, 'deferred_values')
    self.assertListEqual([list(value) for value in values], [['a', 'b']])

    # Iterator > generator.
    values = self.value_expander().Expand(
        self.file, 'imported_dlls.imported_functions')
    expected = [['FindWindow', 'CreateFileA'], ['RegQueryValueEx']]
    self.assertListEqual([list(value) for value in values], expected)

    # Non-existing first path.
    values = self.value_expander().Expand(self.file, 'nonexistant')
    self.assertListEqual(list(values), [])

    # Non-existing in the middle.
    values = self.value_expander().Expand(self.file, 'hash.mink.boo')
    self.assertListEqual(list(values), [])

    # Non-existing as a leaf.
    values = self.value_expander().Expand(self.file, 'hash.mink')
    self.assertListEqual(list(values), [])

    # Non-callable leaf.
    values = self.value_expander().Expand(self.file, 'non_callable_leaf')
    self.assertListEqual(list(values), [DummyFile.non_callable_leaf])

    # callable.
    values = self.value_expander().Expand(self.file, 'Callable')
    self.assertListEqual(list(values), [])

    # leaf under a callable. Will return nothing.
    values = self.value_expander().Expand(self.file, 'Callable.a')
    self.assertListEqual(list(values), [])

  def testGenericBinaryOperator(self):
    class TestBinaryOperator(objectfilter.GenericBinaryOperator):
      values = list()

      def Operation(self, x, _):
        return self.values.append(x)

    # Test a common binary operator.
    tbo = TestBinaryOperator(
        arguments=['whatever', 0], value_expander=self.value_expander)
    self.assertEqual(tbo.right_operand, 0)
    self.assertEqual(tbo.args[0], 'whatever')
    tbo.Matches(DummyObject('whatever', 'id'))
    tbo.Matches(DummyObject('whatever', 'id2'))
    tbo.Matches(DummyObject('whatever', 'bg'))
    tbo.Matches(DummyObject('whatever', 'bg2'))
    self.assertListEqual(tbo.values, ['id', 'id2', 'bg', 'bg2'])

  def testContext(self):
    self.assertRaises(
        objectfilter.InvalidNumberOfOperands, objectfilter.Context,
        arguments=['context'], value_expander=self.value_expander)

    self.assertRaises(
        objectfilter.InvalidNumberOfOperands, objectfilter.Context,
        arguments=[
            'context', objectfilter.Equals(
                arguments=['path', 'value'],
                value_expander=self.value_expander),
            objectfilter.Equals(
                arguments=['another_path', 'value'],
                value_expander=self.value_expander)],
        value_expander=self.value_expander)

    # One imported_dll imports 2 functions AND one imported_dll imports
    # function RegQueryValueEx.
    arguments = [
        objectfilter.Equals(
            arguments=['imported_dlls.num_imported_functions', 1],
            value_expander=self.value_expander),
        objectfilter.Contains(
            arguments=['imported_dlls.imported_functions',
                       'RegQueryValueEx'],
            value_expander=self.value_expander)]
    condition = objectfilter.AndFilter(arguments=arguments)
    # Without context, it matches because both filters match separately.
    self.assertEqual(True, condition.Matches(self.file))

    arguments = [
        objectfilter.Equals(
            arguments=['num_imported_functions', 2],
            value_expander=self.value_expander),
        objectfilter.Contains(
            arguments=['imported_functions', 'RegQueryValueEx'],
            value_expander=self.value_expander)]
    condition = objectfilter.AndFilter(arguments=arguments)
    # The same DLL imports 2 functions AND one of these is RegQueryValueEx.
    context = objectfilter.Context(arguments=['imported_dlls', condition],
                                   value_expander=self.value_expander)
    # With context, it doesn't match because both don't match in the same dll.
    self.assertEqual(False, context.Matches(self.file))

    # One imported_dll imports only 1 function AND one imported_dll imports
    # function RegQueryValueEx.
    condition = objectfilter.AndFilter(arguments=[
        objectfilter.Equals(
            arguments=['num_imported_functions', 1],
            value_expander=self.value_expander),
        objectfilter.Contains(
            arguments=['imported_functions', 'RegQueryValueEx'],
            value_expander=self.value_expander)])
    # The same DLL imports 1 function AND it's RegQueryValueEx.
    context = objectfilter.Context(['imported_dlls', condition],
                                   value_expander=self.value_expander)
    self.assertEqual(True, context.Matches(self.file))

    # Now test the context with a straight query.
    query = u'\n'.join([
        '@imported_dlls',
        '(',
        '  imported_functions contains "RegQueryValueEx"',
        '  AND num_imported_functions == 1',
        ')'])

    filter_ = objectfilter.Parser(query).Parse()
    filter_ = filter_.Compile(self.filter_imp)
    self.assertEqual(True, filter_.Matches(self.file))

  def testRegexpRaises(self):
    with self.assertRaises(ValueError):
      objectfilter.Regexp(
          arguments=['name', 'I [dont compile'],
          value_expander=self.value_expander)

  def testEscaping(self):
    parser = objectfilter.Parser(r'a is "\n"').Parse()
    self.assertEqual(parser.args[0], '\n')
    # Invalid escape sequence.
    parser = objectfilter.Parser(r'a is "\z"')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    # Can escape the backslash.
    parser = objectfilter.Parser(r'a is "\\"').Parse()
    self.assertEqual(parser.args[0], '\\')

    # Test hexadecimal escaping.

    # This fails as it's not really a hex escaped string.
    parser = objectfilter.Parser(r'a is "\xJZ"')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    # Instead, this is what one should write.
    parser = objectfilter.Parser(r'a is "\\xJZ"').Parse()
    self.assertEqual(parser.args[0], r'\xJZ')
    # Standard hex-escape.
    parser = objectfilter.Parser(r'a is "\x41\x41\x41"').Parse()
    self.assertEqual(parser.args[0], 'AAA')
    # Hex-escape + a character.
    parser = objectfilter.Parser(r'a is "\x414"').Parse()
    self.assertEqual(parser.args[0], r'A4')
    # How to include r'\x41'.
    parser = objectfilter.Parser(r'a is "\\x41"').Parse()
    self.assertEqual(parser.args[0], r'\x41')

  def testParse(self):
    # Arguments are either int, float or quoted string.
    objectfilter.Parser('attribute == 1').Parse()
    objectfilter.Parser('attribute == 0x10').Parse()
    parser = objectfilter.Parser('attribute == 1a')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    objectfilter.Parser('attribute == 1.2').Parse()
    objectfilter.Parser('attribute == \'bla\'').Parse()
    objectfilter.Parser('attribute == "bla"').Parse()
    parser = objectfilter.Parser('something == red')
    self.assertRaises(errors.ParseError, parser.Parse)

    # Can't start with AND.
    parser = objectfilter.Parser('and something is \'Blue\'')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    # Test negative filters.
    parser = objectfilter.Parser('attribute not == \'dancer\'')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    parser = objectfilter.Parser('attribute == not \'dancer\'')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    parser = objectfilter.Parser('attribute not not equals \'dancer\'')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    parser = objectfilter.Parser('attribute not > 23')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    # Need to close braces.
    objectfilter.Parser('(a is 3)').Parse()
    parser = objectfilter.Parser('(a is 3')
    self.assertRaises(errors.ParseError, parser.Parse)
    # Need to open braces to close them.
    parser = objectfilter.Parser('a is 3)')
    self.assertRaises(errors.ParseError, parser.Parse)

    # Context Operator alone is not accepted.
    parser = objectfilter.Parser('@attributes')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    # Accepted only with braces.
    objectfilter.Parser('@attributes( name is \'adrien\')').Parse()
    # Not without them.
    parser = objectfilter.Parser('@attributes name is \'adrien\'')
    with self.assertRaises(errors.ParseError):
      parser.Parse()

    # Can nest context operators.
    query = '@imported_dlls( @imported_function( name is \'OpenFileA\'))'
    objectfilter.Parser(query).Parse()
    # Can nest context operators and mix braces without it messing up.
    query = '@imported_dlls( @imported_function( name is \'OpenFileA\'))'
    parser = objectfilter.Parser(query).Parse()
    query = u'\n'.join([
        '@imported_dlls',
        '(',
        '  @imported_function',
        '  (',
        '    name is "OpenFileA" and ordinal == 12',
        '  )',
        ')'])

    parser = objectfilter.Parser(query).Parse()
    # Mix context and binary operators.
    query = u'\n'.join([
        '@imported_dlls',
        '(',
        '  @imported_function',
        '  (',
        '    name is "OpenFileA"',
        '  ) AND num_functions == 2',
        ')'])

    parser = objectfilter.Parser(query).Parse()
    # Also on the right.
    query = u'\n'.join([
        '@imported_dlls',
        '(',
        '  num_functions == 2 AND',
        '  @imported_function',
        '  (',
        '    name is "OpenFileA"',
        '  )',
        ')'])

  # Altogether.
  # There's an imported dll that imports OpenFileA AND
  # an imported DLL matching advapi32.dll that imports RegQueryValueExA AND
  # and it exports a symbol called 'inject'.
  query = u'\n'.join([
      '@imported_dlls( @imported_function ( name is "OpenFileA" ) )',
      'AND',
      '@imported_dlls (',
      '  name regexp "(?i)advapi32.dll"',
      '  AND @imported_function ( name is "RegQueryValueEx" )',
      ')',
      'AND @exported_symbols(name is "inject")'])

  def testCompile(self):
    obj = DummyObject('something', 'Blue')
    parser = objectfilter.Parser('something == \'Blue\'').Parse()
    filter_ = parser.Compile(self.filter_imp)
    self.assertEqual(filter_.Matches(obj), True)
    parser = objectfilter.Parser('something == \'Red\'').Parse()
    filter_ = parser.Compile(self.filter_imp)
    self.assertEqual(filter_.Matches(obj), False)
    parser = objectfilter.Parser('something == "Red"').Parse()
    filter_ = parser.Compile(self.filter_imp)
    self.assertEqual(filter_.Matches(obj), False)
    obj = DummyObject('size', 4)
    parser = objectfilter.Parser('size < 3').Parse()
    filter_ = parser.Compile(self.filter_imp)
    self.assertEqual(filter_.Matches(obj), False)
    parser = objectfilter.Parser('size == 4').Parse()
    filter_ = parser.Compile(self.filter_imp)
    self.assertEqual(filter_.Matches(obj), True)
    query = 'something is \'Blue\' and size not contains 3'
    parser = objectfilter.Parser(query).Parse()
    filter_ = parser.Compile(self.filter_imp)
    self.assertEqual(filter_.Matches(obj), False)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for Android WebviewCache databases."""

from dfdatetime import java_time as dfdatetime_java_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class AndroidWebViewCacheEventData(events.EventData):
  """Android WebViewCache event data.

  Attributes:
    content_length (int): size of the cached content.
    url (str): URL the content was retrieved from.
  """

  DATA_TYPE = u'android:webviewcache'

  def __init__(self):
    """Initializes event data."""
    super(AndroidWebViewCacheEventData, self).__init__(data_type=self.DATA_TYPE)
    self.content_length = None
    self.url = None


class AndroidWebViewCachePlugin(interface.SQLitePlugin):
  """Parser for Android WebViewCache databases."""

  NAME = u'android_webviewcache'
  DESCRIPTION = u'Parser for Android WebViewCache databases'

  REQUIRED_TABLES = frozenset([u'android_metadata', u'cache'])

  QUERIES = frozenset([
      (u'SELECT url, contentlength, expires, lastmodify FROM cache',
       u'ParseRow')])

  SCHEMAS = [{
      u'android_metadata': (
          u'CREATE TABLE android_metadata (locale TEXT)'),
      u'cache': (
          u'CREATE TABLE cache (_id INTEGER PRIMARY KEY, url TEXT, filepath '
          u'TEXT, lastmodify TEXT, etag TEXT, expires INTEGER, expiresstring '
          u'TEXT, mimetype TEXT, encoding TEXT, httpstatus INTEGER, location '
          u'TEXT, contentlength INTEGER, contentdisposition TEXT, UNIQUE (url) '
          u'ON CONFLICT REPLACE)')}]

  def ParseRow(self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses a row from the database.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string']
    # and will raise "IndexError: Index must be int or string". All indexes are
    # thus raw strings.
    event_data = AndroidWebViewCacheEventData()
    event_data.content_length = row['contentlength']
    event_data.query = query
    event_data.url = row['url']

    if row['expires'] is not None:
      date_time = dfdatetime_java_time.JavaTime(timestamp=row['expires'])
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_EXPIRATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    if row['lastmodify'] is not None:
      date_time = dfdatetime_java_time.JavaTime(timestamp=row['lastmodify'])
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(AndroidWebViewCachePlugin)



# -*- coding: utf-8 -*-
"""The Systemd journal file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class SystemdJournalEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Systemd journal event."""

  DATA_TYPE = u'systemd:journal'

  # It would be nice to have the _MACHINE_ID field, which is a unique identifier
  # for the system, and hopefully more unique than the _HOSTNAME field.
  # Unfortunately, journal files that have not been closed cleanly may contain
  # entries that have no _MACHINE_ID field.

  FORMAT_STRING_SEPARATOR = u''

  FORMAT_STRING_PIECES = [
      u'{hostname} ',
      u'[',
      u'{reporter}',
      u', pid: {pid}',
      u'] {body}']

  SOURCE_LONG = u'systemd-journal'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(SystemdJournalEventFormatter)



# -*- coding: utf-8 -*-
"""The Safari history event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class SafariHistoryFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Safari history event."""

  DATA_TYPE = u'safari:history:visit'

  FORMAT_STRING_PIECES = [
      u'Visited: {url}',
      u'({title}',
      u'- {display_title}',
      u')',
      u'Visit Count: {visit_count}']

  SOURCE_LONG = u'Safari History'
  SOURCE_SHORT = u'WEBHIST'


manager.FormattersManager.RegisterFormatter(SafariHistoryFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the event formatters interface classes."""

import unittest

from plaso.formatters import interface
from plaso.formatters import mediator

from tests.containers import test_lib as containers_test_lib
from tests.formatters import test_lib


class BrokenConditionalEventFormatter(interface.ConditionalEventFormatter):
  """An event object for testing the conditional event formatter."""
  DATA_TYPE = u'test:broken_conditional'
  FORMAT_STRING_PIECES = [u'{too} {many} formatting placeholders']

  SOURCE_SHORT = u'LOG'
  SOURCE_LONG = u'Some Text File.'


class ConditionalTestEvent(containers_test_lib.TestEvent):
  """An event object for testing the conditional event formatter."""
  DATA_TYPE = u'test:event:conditional'


class ConditionalTestEventFormatter(interface.ConditionalEventFormatter):
  """An event formatter for testing the conditional event formatter."""
  DATA_TYPE = u'test:event:conditional'
  FORMAT_STRING_PIECES = [
      u'Description: {description}',
      u'Comment',
      u'Value: 0x{numeric:02x}',
      u'Optional: {optional}',
      u'Text: {text}']

  SOURCE_SHORT = u'LOG'
  SOURCE_LONG = u'Some Text File.'


class WrongEventFormatter(interface.EventFormatter):
  """An event formatter for testing."""
  DATA_TYPE = u'test:wrong'
  FORMAT_STRING = u'This format string does not match {body}.'

  SOURCE_SHORT = u'FILE'
  SOURCE_LONG = u'Weird Log File'


class EventFormatterTest(unittest.TestCase):
  """Tests for the event formatter."""

  def setUp(self):
    """Makes preparations before running an individual test."""
    self._event_objects = containers_test_lib.CreateTestEvents()

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = test_lib.TestEventFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = test_lib.TestEventFormatter()

    expected_attribute_names = [u'text']

    attribute_names = event_formatter.GetFormatStringAttributeNames()
    self.assertEqual(sorted(attribute_names), expected_attribute_names)

  # TODO: add test for GetMessages.
  # TODO: add test for GetSources.


class ConditionalEventFormatterTest(unittest.TestCase):
  """Tests for the conditional event formatter."""

  def setUp(self):
    """Makes preparations before running an individual test."""
    self._event_object = ConditionalTestEvent(1335791207939596, {
        u'numeric': 12, u'description': u'this is beyond words',
        u'text': u'but we\'re still trying to say something about the event'})

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = ConditionalTestEventFormatter()
    self.assertIsNotNone(event_formatter)

    with self.assertRaises(RuntimeError):
      _ = BrokenConditionalEventFormatter()

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = ConditionalTestEventFormatter()

    expected_attribute_names = sorted([
        u'description', u'numeric', u'optional', u'text'])

    attribute_names = event_formatter.GetFormatStringAttributeNames()
    self.assertEqual(sorted(attribute_names), expected_attribute_names)

  def testGetMessages(self):
    """Tests the GetMessages function."""
    formatter_mediator = mediator.FormatterMediator()
    event_formatter = ConditionalTestEventFormatter()

    expected_message = (
        u'Description: this is beyond words Comment Value: 0x0c '
        u'Text: but we\'re still trying to say something about the event')

    message, _ = event_formatter.GetMessages(
        formatter_mediator, self._event_object)
    self.assertEqual(message, expected_message)

  # TODO: add test for GetSources.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains a Symantec parser in plaso."""

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import manager
from plaso.parsers import text_parser

import pytz  # pylint: disable=wrong-import-order


__author__ = 'David Nides (david.nides@gmail.com)'


class SymantecEventData(events.EventData):
  """Symantec event data.

  Attributes:
    access (str): access.
    action0 (str): action0.
    action1 (str): action1.
    action1_status (str): action1 status.
    action2 (str): action2.
    action2_status (str): action2 status.
    address (str): address.
    backup_id (str): backup id.
    cat (str): cat.
    cleaninfo (str): cleaninfo.
    clientgroup (str): clientgroup.
    compressed (str): compressed.
    computer (str): computer.
    definfo (str): definfo.
    defseqnumber (str): defseqnumber.
    deleteinfo (str): deleteinfo.
    depth (str): depth.
    description (str): description.
    domain_guid (str): domain guid.
    domainname (str): domainname.
    err_code (str): err code.
    event_data (str): event data.
    event (str): event.
    extra (str): extra.
    file (str): file.
    flags (str): flags.
    groupid (str): groupid.
    guid (str): guid.
    license_expiration_dt (str): license expiration dt.
    license_feature_name (str): license feature name.
    license_feature_ver (str): license feature ver.
    license_fulfillment_id (str): license fulfillment id.
    license_lifecycle (str): license lifecycle.
    license_seats_delta (str): license seats delta.
    license_seats (str): license seats.
    license_seats_total (str): license seats total.
    license_serial_num (str): license serial num.
    license_start_dt (str): license start dt.
    logger (str): logger.
    login_domain (str): login domain.
    log_session_guid (str): log session guid.
    macaddr (str): macaddr.
    new_ext (str): new ext.
    ntdomain (str): ntdomain.
    offset (str): offset.
    parent (str): parent.
    quarfwd_status (str): quarfwd status.
    remote_machine_ip (str): remote machine ip.
    remote_machine (str): remote machine.
    scanid (str): scanid.
    snd_status (str): snd status.
    status (str): status.
    still_infected (str): still infected.
    time (str): time.
    user (str): user.
    vbin_id (str): vbin id.
    vbin_session_id (str): vbin session id.
    version (str): version.
    virus_id (str): virus id.
    virus (str): virus.
    virustype (str): virustype.
  """

  DATA_TYPE = u'av:symantec:scanlog'

  def __init__(self):
    """Initializes event data."""
    super(SymantecEventData, self).__init__(data_type=self.DATA_TYPE)
    self.access = None
    self.action0 = None
    self.action1 = None
    self.action1_status = None
    self.action2 = None
    self.action2_status = None
    self.address = None
    self.backup_id = None
    self.cat = None
    self.cleaninfo = None
    self.clientgroup = None
    self.compressed = None
    self.computer = None
    self.definfo = None
    self.defseqnumber = None
    self.deleteinfo = None
    self.depth = None
    self.description = None
    self.domain_guid = None
    self.domainname = None
    self.err_code = None
    self.event_data = None
    self.event = None
    self.extra = None
    self.file = None
    self.flags = None
    self.groupid = None
    self.guid = None
    self.license_expiration_dt = None
    self.license_feature_name = None
    self.license_feature_ver = None
    self.license_fulfillment_id = None
    self.license_lifecycle = None
    self.license_seats_delta = None
    self.license_seats = None
    self.license_seats_total = None
    self.license_serial_num = None
    self.license_start_dt = None
    self.logger = None
    self.login_domain = None
    self.log_session_guid = None
    self.macaddr = None
    self.new_ext = None
    self.ntdomain = None
    self.offset = None
    self.parent = None
    self.quarfwd_status = None
    self.remote_machine_ip = None
    self.remote_machine = None
    self.scanid = None
    self.snd_status = None
    self.status = None
    self.still_infected = None
    self.time = None
    self.user = None
    self.vbin_id = None
    self.vbin_session_id = None
    self.version = None
    self.virus_id = None
    self.virus = None
    self.virustype = None


class SymantecParser(text_parser.TextCSVParser):
  """Parse Symantec AV Corporate Edition and Endpoint Protection log files."""

  NAME = u'symantec_scanlog'
  DESCRIPTION = u'Parser for Symantec Anti-Virus log files.'

  # Define the columns that make up the structure of a Symantec log file.
  # http://www.symantec.com/docs/TECH100099
  COLUMNS = [
      u'time', u'event', u'cat', u'logger', u'computer', u'user',
      u'virus', u'file', u'action1', u'action2', u'action0', u'virustype',
      u'flags', u'description', u'scanid', u'new_ext', u'groupid',
      u'event_data', u'vbin_id', u'virus_id', u'quarfwd_status',
      u'access', u'snd_status', u'compressed', u'depth', u'still_infected',
      u'definfo', u'defseqnumber', u'cleaninfo', u'deleteinfo',
      u'backup_id', u'parent', u'guid', u'clientgroup', u'address',
      u'domainname', u'ntdomain', u'macaddr', u'version:',
      u'remote_machine', u'remote_machine_ip', u'action1_status',
      u'action2_status', u'license_feature_name', u'license_feature_ver',
      u'license_serial_num', u'license_fulfillment_id', u'license_start_dt',
      u'license_expiration_dt', u'license_lifecycle', u'license_seats_total',
      u'license_seats', u'err_code', u'license_seats_delta', u'status',
      u'domain_guid', u'log_session_guid', u'vbin_session_id',
      u'login_domain', u'extra']

  def _ConvertToTimestamp(self, date_time_values, timezone=pytz.UTC):
    """Converts the given parsed date and time values to a timestamp.

    The date and time values consist of six hexadecimal octets.
    They represent the following:
      First octet: Number of years since 1970
      Second octet: Month, where January = 0
      Third octet: Day
      Fourth octet: Hour
      Fifth octet: Minute
      Sixth octet: Second

    For example, 200A13080122 represents November 19, 2002, 8:01:34 AM.

    Args:
      date_time_values: The hexadecimal encoded date and time values.
      timezone: Optional timezone (instance of pytz.timezone).

    Returns:
      A timestamp value, contains the number of micro seconds since
      January 1, 1970, 00:00:00 UTC.
    """
    if not date_time_values:
      return timelib.Timestamp.NONE_TIMESTAMP

    year, month, day, hours, minutes, seconds = (
        int(hexdigit[0] + hexdigit[1], 16) for hexdigit in zip(
            date_time_values[::2], date_time_values[1::2]))

    return timelib.Timestamp.FromTimeParts(
        year + 1970, month + 1, day, hours, minutes, seconds, timezone=timezone)

  def ParseRow(self, parser_mediator, row_offset, row):
    """Parses a row and extract event objects.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row_offset (int): line number of the row.
      row (dict[str, str]): row of the fields specified in COLUMNS.
    """
    try:
      timestamp = self._ConvertToTimestamp(
          row[u'time'], timezone=parser_mediator.timezone)
    except (TypeError, ValueError, errors.TimestampError) as exception:
      timestamp = timelib.Timestamp.NONE_TIMESTAMP
      parser_mediator.ProduceExtractionError(
          u'unable to determine timestamp with error: {0:s}'.format(
              exception))

    # TODO: remove unused attributes.
    event_data = SymantecEventData()
    event_data.access = row.get(u'access', None)
    event_data.action0 = row.get(u'action0', None)
    event_data.action1 = row.get(u'action1', None)
    event_data.action1_status = row.get(u'action1_status', None)
    event_data.action2 = row.get(u'action2', None)
    event_data.action2_status = row.get(u'action2_status', None)
    event_data.address = row.get(u'address', None)
    event_data.backup_id = row.get(u'backup_id', None)
    event_data.cat = row.get(u'cat', None)
    event_data.cleaninfo = row.get(u'cleaninfo', None)
    event_data.clientgroup = row.get(u'clientgroup', None)
    event_data.compressed = row.get(u'compressed', None)
    event_data.computer = row.get(u'computer', None)
    event_data.definfo = row.get(u'definfo', None)
    event_data.defseqnumber = row.get(u'defseqnumber', None)
    event_data.deleteinfo = row.get(u'deleteinfo', None)
    event_data.depth = row.get(u'depth', None)
    event_data.description = row.get(u'description', None)
    event_data.domain_guid = row.get(u'domain_guid', None)
    event_data.domainname = row.get(u'domainname', None)
    event_data.err_code = row.get(u'err_code', None)
    event_data.event_data = row.get(u'event_data', None)
    event_data.event = row.get(u'event', None)
    event_data.extra = row.get(u'extra', None)
    event_data.file = row.get(u'file', None)
    event_data.flags = row.get(u'flags', None)
    event_data.groupid = row.get(u'groupid', None)
    event_data.guid = row.get(u'guid', None)
    event_data.license_expiration_dt = row.get(u'license_expiration_dt', None)
    event_data.license_feature_name = row.get(u'license_feature_name', None)
    event_data.license_feature_ver = row.get(u'license_feature_ver', None)
    event_data.license_fulfillment_id = row.get(u'license_fulfillment_id', None)
    event_data.license_lifecycle = row.get(u'license_lifecycle', None)
    event_data.license_seats_delta = row.get(u'license_seats_delta', None)
    event_data.license_seats = row.get(u'license_seats', None)
    event_data.license_seats_total = row.get(u'license_seats_total', None)
    event_data.license_serial_num = row.get(u'license_serial_num', None)
    event_data.license_start_dt = row.get(u'license_start_dt', None)
    event_data.logger = row.get(u'logger', None)
    event_data.login_domain = row.get(u'login_domain', None)
    event_data.log_session_guid = row.get(u'log_session_guid', None)
    event_data.macaddr = row.get(u'macaddr', None)
    event_data.new_ext = row.get(u'new_ext', None)
    event_data.ntdomain = row.get(u'ntdomain', None)
    event_data.offset = row_offset
    event_data.parent = row.get(u'parent', None)
    event_data.quarfwd_status = row.get(u'quarfwd_status', None)
    event_data.remote_machine_ip = row.get(u'remote_machine_ip', None)
    event_data.remote_machine = row.get(u'remote_machine', None)
    event_data.scanid = row.get(u'scanid', None)
    event_data.snd_status = row.get(u'snd_status', None)
    event_data.status = row.get(u'status', None)
    event_data.still_infected = row.get(u'still_infected', None)
    event_data.time = row.get(u'time', None)
    event_data.user = row.get(u'user', None)
    event_data.vbin_id = row.get(u'vbin_id', None)
    event_data.vbin_session_id = row.get(u'vbin_session_id', None)
    event_data.version = row.get(u'version:', None)
    event_data.virus_id = row.get(u'virus_id', None)
    event_data.virus = row.get(u'virus', None)
    event_data.virustype = row.get(u'virustype', None)

    event = time_events.TimestampEvent(
        timestamp, definitions.TIME_DESCRIPTION_WRITTEN)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def VerifyRow(self, parser_mediator, row):
    """Verify a single line of a Symantec log file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (dict[str, str]): row of the fields specified in COLUMNS.

    Returns:
      bool: True if the row is in the expected format, False if not.
    """
    try:
      timestamp = self._ConvertToTimestamp(
          row[u'time'], timezone=parser_mediator.timezone)
    except (TypeError, ValueError, errors.TimestampError):
      return False

    if not timestamp:
      return False

    # Check few entries.
    try:
      my_event = int(row[u'event'])
    except (TypeError, ValueError):
      return False

    if my_event < 1 or my_event > 77:
      return False

    try:
      category = int(row[u'cat'])
    except (TypeError, ValueError):
      return False

    if category < 1 or category > 4:
      return False

    return True


manager.ParsersManager.RegisterParser(SymantecParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Mac OS X keychain password database file event formatter."""

import unittest

from plaso.formatters import mac_keychain

from tests.formatters import test_lib


class KeychainApplicationRecordFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the keychain application record event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = mac_keychain.KeychainApplicationRecordFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = mac_keychain.KeychainApplicationRecordFormatter()

    expected_attribute_names = [
        u'entry_name',
        u'account_name']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class KeychainInternetRecordFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the keychain Internet record event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = mac_keychain.KeychainInternetRecordFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = mac_keychain.KeychainInternetRecordFormatter()

    expected_attribute_names = [
        u'entry_name',
        u'account_name',
        u'where',
        u'protocol',
        u'type_protocol']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the Basic Security Module definitions."""

# Arbitrary tokens.
# Type of data to print in a BSM_TOKEN_DATA.
BSM_TOKEN_DATA_TYPE = {
    0: u'AUR_CHAR',
    1: u'AUR_SHORT',
    2: u'AUR_INT32'}

BSM_TOKEN_DATA_PRINT = {
    0: u'Binary',
    1: u'Octal',
    2: u'Decimal',
    3: u'Hexadecimal',
    4: u'String'}

# BSM identification errors.
BSM_ERRORS = {
    0: u'Success',
    1: u'Operation not permitted',
    2: u'No such file or directory',
    3: u'No such process',
    4: u'Interrupted system call',
    5: u'Input/output error',
    6: u'Device not configured',
    7: u'Argument list too long',
    8: u'Exec format error',
    9: u'Bad file descriptor',
    10: u'No child processes',
    11: u'Resource temporarily unavailable',
    12: u'Cannot allocate memory',
    13: u'Permission denied',
    14: u'Bad address',
    15: u'Block device required',
    16: u'Device busy',
    17: u'File exists',
    18: u'ross-device link',
    19: u'Operation not supported by device',
    20: u'Not a directory',
    21: u'Is a directory',
    22: u'Invalid argument',
    23: u'Too many open files in system',
    24: u'Too many open files',
    25: u'Inappropriate ioctl for device',
    26: u'Text file busy',
    27: u'File too large',
    28: u'No space left on device',
    29: u'Illegal seek',
    30: u'Read-only file system',
    31: u'Too many links',
    32: u'Broken pipe',
    33: u'Numerical argument out of domain',
    34: u'Result too large',
    35: u'No message of desired type',
    36: u'Identifier removed',
    45: u'Resource deadlock avoided',
    46: u'No locks available',
    47: u'Operation canceled',
    48: u'Operation not supported',
    49: u'Disc quota exceeded',
    66: u'Too many levels of remote in path',
    67: u'Link has been severed',
    71: u'Protocol error',
    74: u'Multihop attempted',
    77: u'Bad message',
    78: u'File name too long',
    79: u'Value too large to be stored in data type',
    88: u'Illegal byte sequence',
    89: u'Function not implemented',
    90: u'Too many levels of symbolic links',
    91: u'Restart syscall',
    93: u'Directory not empty',
    94: u'Too many users',
    95: u'Socket operation on non-socket',
    96: u'Destination address required',
    97: u'Message too long',
    98: u'Protocol wrong type for socket',
    99: u'Protocol not available',
    120: u'Protocol not supported',
    121: u'Socket type not supported',
    122: u'Operation not supported',
    123: u'Protocol family not supported',
    124: u'Address family not supported by protocol family',
    125: u'Address already in use',
    126: u'Can\'t assign requested address',
    127: u'Network is down',
    128: u'Network unreachable',
    129: u'Network dropped connection on reset',
    130: u'Software caused connection abort',
    131: u'Connection reset by peer',
    132: u'No buffer space available',
    133: u'Socket is already connected',
    134: u'Socket is not connected',
    143: u'Can\'t send after socket shutdown',
    144: u'Too many references: can\'t splice',
    145: u'Operation timed out',
    146: u'Connection refused',
    147: u'Host is down',
    148: u'No route to host',
    149: u'Operation already in progress',
    150: u'Operation now in progress',
    151: u'Stale NFS file handle',
    190: u'PROCLIM',
    191: u'BADRPC',
    192: u'RPCMISMATCH',
    193: u'PROGUNAVAIL',
    194: u'PROGMISMATCH',
    195: u'PROCUNAVAIL',
    196: u'FTYPE',
    197: u'AUTH',
    198: u'NEEDAUTH',
    199: u'NOATTR',
    200: u'DOOFUS',
    201: u'USTRETURN',
    202: u'NOIOCTL',
    203: u'DIRIOCTL',
    204: u'PWROFF',
    205: u'DEVERR',
    206: u'BADEXEC',
    207: u'BADARCH',
    208: u'SHLIBVERS',
    209: u'BADMACHO',
    210: u'POLICY'}

# BSM network protocols. The informations comes from OpenBSD project,
# it might not be exacly.
BSM_PROTOCOLS = {
    0: u'UNSPEC',
    1: u'LOCAL',
    2: u'INET',
    3: u'IMPLINK',
    4: u'PUP',
    5: u'CHAOS',
    6: u'NS',
    8: u'ECMA',
    9: u'DATAKIT',
    10: u'CCITT',
    11: u'SNA',
    12: u'DECnet',
    13: u'DLI',
    14: u'LAT',
    15: u'HYLINK',
    16: u'APPLETALK',
    19: u'OSI',
    23: u'IPX',
    24: u'ROUTE',
    25: u'LINK',
    26: u'INET6',
    27: u'KEY',
    500: u'NETBIOS',
    501: u'ISO',
    502: u'XTP',
    503: u'COIP',
    504: u'CNT',
    505: u'RTIP',
    506: u'SIP',
    507: u'PIP',
    508: u'ISDN',
    509: u'E164',
    510: u'NATM',
    511: u'ATM',
    512: u'NETGRAPH',
    513: u'SLOW',
    514: u'CLUSTER',
    515: u'ARP',
    516: u'BLUETOOTH'}

# key event types.
BSM_AUDIT_EVENT = {
    0: u'indir system call',
    1: u'exit(2)',
    2: u'fork(2)',
    3: u'open(2) - attr only',
    4: u'creat(2)',
    5: u'link(2)',
    6: u'unlink(2)',
    7: u'exec(2)',
    8: u'chdir(2)',
    9: u'mknod(2)',
    10: u'chmod(2)',
    11: u'chown(2)',
    12: u'umount(2) - old version',
    13: u'junk',
    14: u'access(2)',
    15: u'kill(2)',
    16: u'stat(2)',
    17: u'lstat(2)',
    18: u'acct(2)',
    19: u'mctl(2)',
    20: u'reboot(2)',
    21: u'symlink(2)',
    22: u'readlink(2)',
    23: u'execve(2)',
    24: u'chroot(2)',
    25: u'vfork(2)',
    26: u'setgroups(2)',
    27: u'setpgrp(2)',
    28: u'swapon(2)',
    29: u'sethostname(2)',
    30: u'fcntl(2)',
    31: u'setpriority(2)',
    32: u'connect(2)',
    33: u'accept(2)',
    34: u'bind(2)',
    35: u'setsockopt(2)',
    36: u'vtrace(2)',
    37: u'settimeofday(2)',
    38: u'fchown(2)',
    39: u'fchmod(2)',
    40: u'setreuid(2)',
    41: u'setregid(2)',
    42: u'rename(2)',
    43: u'truncate(2)',
    44: u'ftruncate(2)',
    45: u'flock(2)',
    46: u'shutdown(2)',
    47: u'mkdir(2)',
    48: u'rmdir(2)',
    49: u'utimes(2)',
    50: u'adjtime(2)',
    51: u'setrlimit(2)',
    52: u'killpg(2)',
    53: u'nfs_svc(2)',
    54: u'statfs(2)',
    55: u'fstatfs(2)',
    56: u'unmount(2)',
    57: u'async_daemon(2)',
    58: u'nfs_getfh(2)',
    59: u'setdomainname(2)',
    60: u'quotactl(2)',
    61: u'exportfs(2)',
    62: u'mount(2)',
    63: u'semsys(2)',
    64: u'msgsys(2)',
    65: u'shmsys(2)',
    66: u'bsmsys(2)',
    67: u'rfssys(2)',
    68: u'fchdir(2)',
    69: u'fchroot(2)',
    70: u'vpixsys(2)',
    71: u'pathconf(2)',
    72: u'open(2) - read',
    73: u'open(2) - read,creat',
    74: u'open(2) - read,trunc',
    75: u'open(2) - read,creat,trunc',
    76: u'open(2) - write',
    77: u'open(2) - write,creat',
    78: u'open(2) - write,trunc',
    79: u'open(2) - write,creat,trunc',
    80: u'open(2) - read,write',
    81: u'open(2) - read,write,creat',
    82: u'open(2) - read,write,trunc',
    83: u'open(2) - read,write,creat,trunc',
    84: u'msgctl(2) - illegal command',
    85: u'msgctl(2) - IPC_RMID command',
    86: u'msgctl(2) - IPC_SET command',
    87: u'msgctl(2) - IPC_STAT command',
    88: u'msgget(2)',
    89: u'msgrcv(2)',
    90: u'msgsnd(2)',
    91: u'shmctl(2) - illegal command',
    92: u'shmctl(2) - IPC_RMID command',
    93: u'shmctl(2) - IPC_SET command',
    94: u'shmctl(2) - IPC_STAT command',
    95: u'shmget(2)',
    96: u'shmat(2)',
    97: u'shmdt(2)',
    98: u'semctl(2) - illegal command',
    99: u'semctl(2) - IPC_RMID command',
    100: u'semctl(2) - IPC_SET command',
    101: u'semctl(2) - IPC_STAT command',
    102: u'semctl(2) - GETNCNT command',
    103: u'semctl(2) - GETPID command',
    104: u'semctl(2) - GETVAL command',
    105: u'semctl(2) - GETALL command',
    106: u'semctl(2) - GETZCNT command',
    107: u'semctl(2) - SETVAL command',
    108: u'semctl(2) - SETALL command',
    109: u'semget(2)',
    110: u'semop(2)',
    111: u'process dumped core',
    112: u'close(2)',
    113: u'system booted',
    114: u'async_daemon(2) exited',
    115: u'nfssvc(2) exited',
    128: u'writel(2)',
    129: u'writevl(2)',
    130: u'getauid(2)',
    131: u'setauid(2)',
    132: u'getaudit(2)',
    133: u'setaudit(2)',
    134: u'getuseraudit(2)',
    135: u'setuseraudit(2)',
    136: u'auditsvc(2)',
    137: u'audituser(2)',
    138: u'auditon(2)',
    139: u'auditon(2) - GETTERMID command',
    140: u'auditon(2) - SETTERMID command',
    141: u'auditon(2) - GPOLICY command',
    142: u'auditon(2) - SPOLICY command',
    143: u'auditon(2) - GESTATE command',
    144: u'auditon(2) - SESTATE command',
    145: u'auditon(2) - GQCTRL command',
    146: u'auditon(2) - SQCTRL command',
    147: u'getkernstate(2)',
    148: u'setkernstate(2)',
    149: u'getportaudit(2)',
    150: u'auditstat(2)',
    151: u'revoke(2)',
    152: u'Solaris AUE_MAC',
    153: u'enter prom',
    154: u'exit prom',
    155: u'Solaris AUE_IFLOAT',
    156: u'Solaris AUE_PFLOAT',
    157: u'Solaris AUE_UPRIV',
    158: u'ioctl(2)',
    173: u'one-sided session record',
    174: u'msggetl(2)',
    175: u'msgrcvl(2)',
    176: u'msgsndl(2)',
    177: u'semgetl(2)',
    178: u'shmgetl(2)',
    183: u'socket(2)',
    184: u'sendto(2)',
    185: u'pipe(2)',
    186: u'socketpair(2)',
    187: u'send(2)',
    188: u'sendmsg(2)',
    189: u'recv(2)',
    190: u'recvmsg(2)',
    191: u'recvfrom(2)',
    192: u'read(2)',
    193: u'getdents(2)',
    194: u'lseek(2)',
    195: u'write(2)',
    196: u'writev(2)',
    197: u'nfs server',
    198: u'readv(2)',
    199: u'Solaris old stat(2)',
    200: u'setuid(2)',
    201: u'old stime(2)',
    202: u'old utime(2)',
    203: u'old nice(2)',
    204: u'Solaris old setpgrp(2)',
    205: u'setgid(2)',
    206: u'readl(2)',
    207: u'readvl(2)',
    208: u'fstat(2)',
    209: u'dup2(2)',
    210: u'mmap(2)',
    211: u'audit(2)',
    212: u'Solaris priocntlsys(2)',
    213: u'munmap(2)',
    214: u'setegid(2)',
    215: u'seteuid(2)',
    216: u'putmsg(2)',
    217: u'getmsg(2)',
    218: u'putpmsg(2)',
    219: u'getpmsg(2)',
    220: u'audit system calls place holder',
    221: u'auditon(2) - get kernel mask',
    222: u'auditon(2) - set kernel mask',
    223: u'auditon(2) - get cwd',
    224: u'auditon(2) - get car',
    225: u'auditon(2) - get audit statistics',
    226: u'auditon(2) - reset audit statistics',
    227: u'auditon(2) - set mask per uid',
    228: u'auditon(2) - set mask per session ID',
    229: u'auditon(2) - get audit state',
    230: u'auditon(2) - set audit state',
    231: u'auditon(2) - get event class',
    232: u'auditon(2) - set event class',
    233: u'utssys(2) - fusers',
    234: u'statvfs(2)',
    235: u'xstat(2)',
    236: u'lxstat(2)',
    237: u'lchown(2)',
    238: u'memcntl(2)',
    239: u'sysinfo(2)',
    240: u'xmknod(2)',
    241: u'fork1(2)',
    242: u'modctl(2) system call place holder',
    243: u'modctl(2) - load module',
    244: u'modctl(2) - unload module',
    245: u'modctl(2) - configure module',
    246: u'modctl(2) - bind module',
    247: u'getmsg-accept',
    248: u'putmsg-connect',
    249: u'putmsg-send',
    250: u'getmsg-receive',
    251: u'acl(2) - SETACL comand',
    252: u'facl(2) - SETACL command',
    253: u'doorfs(2) - system call place holder',
    254: u'doorfs(2) - DOOR_CALL',
    255: u'doorfs(2) - DOOR_RETURN',
    256: u'doorfs(2) - DOOR_CREATE',
    257: u'doorfs(2) - DOOR_REVOKE',
    258: u'doorfs(2) - DOOR_INFO',
    259: u'doorfs(2) - DOOR_CRED',
    260: u'doorfs(2) - DOOR_BIND',
    261: u'doorfs(2) - DOOR_UNBIND',
    262: u'p_online(2)',
    263: u'processor_bind(2)',
    264: u'inst_sync(2)',
    265: u'configure socket',
    266: u'setaudit_addr(2)',
    267: u'getaudit_addr(2)',
    268: u'Solaris umount(2)',
    269: u'fsat(2) - place holder',
    270: u'openat(2) - read',
    271: u'openat(2) - read,creat',
    272: u'openat(2) - read,trunc',
    273: u'openat(2) - read,creat,trunc',
    274: u'openat(2) - write',
    275: u'openat(2) - write,creat',
    276: u'openat(2) - write,trunc',
    277: u'openat(2) - write,creat,trunc',
    278: u'openat(2) - read,write',
    279: u'openat(2) - read,write,create',
    280: u'openat(2) - read,write,trunc',
    281: u'openat(2) - read,write,creat,trunc',
    282: u'renameat(2)',
    283: u'fstatat(2)',
    284: u'fchownat(2)',
    285: u'futimesat(2)',
    286: u'unlinkat(2)',
    287: u'clock_settime(2)',
    288: u'ntp_adjtime(2)',
    289: u'setppriv(2)',
    290: u'modctl(2) - configure device policy',
    291: u'modctl(2) - configure additional privilege',
    292: u'kernel cryptographic framework',
    293: u'configure kernel SSL',
    294: u'brandsys(2)',
    295: u'Add IPsec policy rule',
    296: u'Delete IPsec policy rule',
    297: u'Clone IPsec policy',
    298: u'Flip IPsec policy',
    299: u'Flush IPsec policy rules',
    300: u'Update IPsec algorithms',
    301: u'portfs',
    302: u'ptrace(2)',
    303: u'chflags(2)',
    304: u'fchflags(2)',
    305: u'profil(2)',
    306: u'ktrace(2)',
    307: u'setlogin(2)',
    308: u'reboot(2)',
    309: u'revoke(2)',
    310: u'umask(2)',
    311: u'mprotect(2)',
    312: u'setpriority(2)',
    313: u'settimeofday(2)',
    314: u'flock(2)',
    315: u'mkfifo(2)',
    316: u'poll(2)',
    317: u'socketpair(2)',
    318: u'futimes(2)',
    319: u'setsid(2)',
    320: u'setprivexec(2)',
    321: u'nfssvc(2)',
    322: u'getfh(2)',
    323: u'quotactl(2)',
    324: u'add_profil()',
    325: u'kdebug_trace()',
    326: u'fstat(2)',
    327: u'fpathconf(2)',
    328: u'getdirentries(2)',
    329: u'truncate(2)',
    330: u'ftruncate(2)',
    331: u'sysctl(3)',
    332: u'mlock(2)',
    333: u'munlock(2)',
    334: u'undelete(2)',
    335: u'getattrlist()',
    336: u'setattrlist()',
    337: u'getdirentriesattr()',
    338: u'exchangedata()',
    339: u'searchfs()',
    340: u'minherit(2)',
    341: u'semconfig()',
    342: u'sem_open(2)',
    343: u'sem_close(2)',
    344: u'sem_unlink(2)',
    345: u'shm_open(2)',
    346: u'shm_unlink(2)',
    347: u'load_shared_file()',
    348: u'reset_shared_file()',
    349: u'new_system_share_regions()',
    350: u'pthread_kill(2)',
    351: u'pthread_sigmask(2)',
    352: u'auditctl(2)',
    353: u'rfork(2)',
    354: u'lchmod(2)',
    355: u'swapoff(2)',
    356: u'init_process()',
    357: u'map_fd()',
    358: u'task_for_pid()',
    359: u'pid_for_task()',
    360: u'sysctl() - non-admin',
    361: u'copyfile()',
    43001: u'getfsstat(2)',
    43002: u'ptrace(2)',
    43003: u'chflags(2)',
    43004: u'fchflags(2)',
    43005: u'profil(2)',
    43006: u'ktrace(2)',
    43007: u'setlogin(2)',
    43008: u'revoke(2)',
    43009: u'umask(2)',
    43010: u'mprotect(2)',
    43011: u'mkfifo(2)',
    43012: u'poll(2)',
    43013: u'futimes(2)',
    43014: u'setsid(2)',
    43015: u'setprivexec(2)',
    43016: u'add_profil()',
    43017: u'kdebug_trace()',
    43018: u'fstat(2)',
    43019: u'fpathconf(2)',
    43020: u'getdirentries(2)',
    43021: u'sysctl(3)',
    43022: u'mlock(2)',
    43023: u'munlock(2)',
    43024: u'undelete(2)',
    43025: u'getattrlist()',
    43026: u'setattrlist()',
    43027: u'getdirentriesattr()',
    43028: u'exchangedata()',
    43029: u'searchfs()',
    43030: u'minherit(2)',
    43031: u'semconfig()',
    43032: u'sem_open(2)',
    43033: u'sem_close(2)',
    43034: u'sem_unlink(2)',
    43035: u'shm_open(2)',
    43036: u'shm_unlink(2)',
    43037: u'load_shared_file()',
    43038: u'reset_shared_file()',
    43039: u'new_system_share_regions()',
    43040: u'pthread_kill(2)',
    43041: u'pthread_sigmask(2)',
    43042: u'auditctl(2)',
    43043: u'rfork(2)',
    43044: u'lchmod(2)',
    43045: u'swapoff(2)',
    43046: u'init_process()',
    43047: u'map_fd()',
    43048: u'task_for_pid()',
    43049: u'pid_for_task()',
    43050: u'sysctl() - non-admin',
    43051: u'copyfile(2)',
    43052: u'lutimes(2)',
    43053: u'lchflags(2)',
    43054: u'sendfile(2)',
    43055: u'uselib(2)',
    43056: u'getresuid(2)',
    43057: u'setresuid(2)',
    43058: u'getresgid(2)',
    43059: u'setresgid(2)',
    43060: u'wait4(2)',
    43061: u'lgetfh(2)',
    43062: u'fhstatfs(2)',
    43063: u'fhopen(2)',
    43064: u'fhstat(2)',
    43065: u'jail(2)',
    43066: u'eaccess(2)',
    43067: u'kqueue(2)',
    43068: u'kevent(2)',
    43069: u'fsync(2)',
    43070: u'nmount(2)',
    43071: u'bdflush(2)',
    43072: u'setfsuid(2)',
    43073: u'setfsgid(2)',
    43074: u'personality(2)',
    43075: u'getscheduler(2)',
    43076: u'setscheduler(2)',
    43077: u'prctl(2)',
    43078: u'getcwd(2)',
    43079: u'capget(2)',
    43080: u'capset(2)',
    43081: u'pivot_root(2)',
    43082: u'rtprio(2)',
    43083: u'sched_getparam(2)',
    43084: u'sched_setparam(2)',
    43085: u'sched_get_priority_max(2)',
    43086: u'sched_get_priority_min(2)',
    43087: u'sched_rr_get_interval(2)',
    43088: u'acl_get_file(2)',
    43089: u'acl_set_file(2)',
    43090: u'acl_get_fd(2)',
    43091: u'acl_set_fd(2)',
    43092: u'acl_delete_file(2)',
    43093: u'acl_delete_fd(2)',
    43094: u'acl_aclcheck_file(2)',
    43095: u'acl_aclcheck_fd(2)',
    43096: u'acl_get_link(2)',
    43097: u'acl_set_link(2)',
    43098: u'acl_delete_link(2)',
    43099: u'acl_aclcheck_link(2)',
    43100: u'sysarch(2)',
    43101: u'extattrctl(2)',
    43102: u'extattr_get_file(2)',
    43103: u'extattr_set_file(2)',
    43104: u'extattr_list_file(2)',
    43105: u'extattr_delete_file(2)',
    43106: u'extattr_get_fd(2)',
    43107: u'extattr_set_fd(2)',
    43108: u'extattr_list_fd(2)',
    43109: u'extattr_delete_fd(2)',
    43110: u'extattr_get_link(2)',
    43111: u'extattr_set_link(2)',
    43112: u'extattr_list_link(2)',
    43113: u'extattr_delete_link(2)',
    43114: u'kenv(8)',
    43115: u'jail_attach(2)',
    43116: u'sysctl(3)',
    43117: u'linux ioperm',
    43118: u'readdir(3)',
    43119: u'linux iopl',
    43120: u'linux vm86',
    43121: u'mac_get_proc(2)',
    43122: u'mac_set_proc(2)',
    43123: u'mac_get_fd(2)',
    43124: u'mac_get_file(2)',
    43125: u'mac_set_fd(2)',
    43126: u'mac_set_file(2)',
    43127: u'mac_syscall(2)',
    43128: u'mac_get_pid(2)',
    43129: u'mac_get_link(2)',
    43130: u'mac_set_link(2)',
    43131: u'mac_execve(2)',
    43132: u'getpath_fromfd(2)',
    43133: u'getpath_fromaddr(2)',
    43134: u'mq_open(2)',
    43135: u'mq_setattr(2)',
    43136: u'mq_timedreceive(2)',
    43137: u'mq_timedsend(2)',
    43138: u'mq_notify(2)',
    43139: u'mq_unlink(2)',
    43140: u'listen(2)',
    43141: u'mlockall(2)',
    43142: u'munlockall(2)',
    43143: u'closefrom(2)',
    43144: u'fexecve(2)',
    43145: u'faccessat(2)',
    43146: u'fchmodat(2)',
    43147: u'linkat(2)',
    43148: u'mkdirat(2)',
    43149: u'mkfifoat(2)',
    43150: u'mknodat(2)',
    43151: u'readlinkat(2)',
    43152: u'symlinkat(2)',
    43153: u'mac_getfsstat(2)',
    43154: u'mac_get_mount(2)',
    43155: u'mac_get_lcid(2)',
    43156: u'mac_get_lctx(2)',
    43157: u'mac_set_lctx(2)',
    43158: u'mac_mount(2)',
    43159: u'getlcid(2)',
    43160: u'setlcid(2)',
    43161: u'taskname_for_pid()',
    43162: u'access_extended(2)',
    43163: u'chmod_extended(2)',
    43164: u'fchmod_extended(2)',
    43165: u'fstat_extended(2)',
    43166: u'lstat_extended(2)',
    43167: u'mkdir_extended(2)',
    43168: u'mkfifo_extended(2)',
    43169: u'open_extended(2) - attr only',
    43170: u'open_extended(2) - read',
    43171: u'open_extended(2) - read,creat',
    43172: u'open_extended(2) - read,trunc',
    43173: u'open_extended(2) - read,creat,trunc',
    43174: u'open_extended(2) - write',
    43175: u'open_extended(2) - write,creat',
    43176: u'open_extended(2) - write,trunc',
    43177: u'open_extended(2) - write,creat,trunc',
    43178: u'open_extended(2) - read,write',
    43179: u'open_extended(2) - read,write,creat',
    43180: u'open_extended(2) - read,write,trunc',
    43181: u'open_extended(2) - read,write,creat,trunc',
    43182: u'stat_extended(2)',
    43183: u'umask_extended(2)',
    43184: u'openat(2) - attr only',
    43185: u'posix_openpt(2)',
    43186: u'cap_new(2)',
    43187: u'cap_getrights(2)',
    43188: u'cap_enter(2)',
    43189: u'cap_getmode(2)',
    43190: u'posix_spawn(2)',
    43191: u'fsgetpath(2)',
    43192: u'pread(2)',
    43193: u'pwrite(2)',
    43194: u'fsctl()',
    43195: u'ffsctl()',
    43196: u'lpathconf(2)',
    43197: u'pdfork(2)',
    43198: u'pdkill(2)',
    43199: u'pdgetpid(2)',
    43200: u'pdwait(2)',
    44901: u'session start',
    44902: u'session update',
    44903: u'session end',
    44904: u'session close',
    6144: u'at-create atjob',
    6145: u'at-delete atjob (at or atrm)',
    6146: u'at-permission',
    6147: u'cron-invoke',
    6148: u'crontab-crontab created',
    6149: u'crontab-crontab deleted',
    6150: u'crontab-permission',
    6151: u'inetd connection',
    6152: u'login - local',
    6153: u'logout - local',
    6154: u'login - telnet',
    6155: u'login - rlogin',
    6156: u'mount',
    6157: u'unmount',
    6158: u'rsh access',
    6159: u'su(1)',
    6160: u'system halt',
    6161: u'system reboot',
    6162: u'rexecd',
    6163: u'passwd',
    6164: u'rexd',
    6165: u'ftp access',
    6166: u'init',
    6167: u'uadmin',
    6168: u'system shutdown',
    6170: u'crontab-modify',
    6171: u'ftp logout',
    6172: u'login - ssh',
    6173: u'role login',
    6180: u' profile command',
    6181: u'add filesystem',
    6182: u'delete filesystem',
    6183: u'modify filesystem',
    6200: u'allocate-device success',
    6201: u'allocate-device failure',
    6202: u'deallocate-device success',
    6203: u'deallocate-device failure',
    6204: u'allocate-list devices success',
    6205: u'allocate-list devices failure',
    6207: u'create user',
    6208: u'modify user',
    6209: u'delete user',
    6210: u'disable user',
    6211: u'enable user',
    6212: u'newgrp login',
    6213: u'admin login',
    6214: u'authenticated kadmind request',
    6215: u'unauthenticated kadmind req',
    6216: u'kdc authentication svc request',
    6217: u'kdc tkt-grant svc request',
    6218: u'kdc tgs 2ndtkt mismtch',
    6219: u'kdc tgs issue alt tgt',
    6300: u'sudo(1)',
    6501: u'modify password',
    6511: u'create group',
    6512: u'delete group',
    6513: u'modify group',
    6514: u'add to group',
    6515: u'remove from group',
    6521: u'revoke object priv',
    6600: u'loginwindow login',
    6601: u'loginwindow logout',
    7000: u'user authentication',
    7001: u'SecSrvr connection setup',
    7002: u'SecSrvr AuthEngine',
    7003: u'SecSrvr authinternal mech',
    32800: u'OpenSSH login',
    45000: u'audit startup',
    45001: u'audit shutdown',
    45014: u'modify password',
    45015: u'create group',
    45016: u'delete group',
    45017: u'modify group',
    45018: u'add to group',
    45019: u'remove from group',
    45020: u'revoke object priv',
    45021: u'loginwindow login',
    45022: u'loginwindow logout',
    45023: u'user authentication',
    45024: u'SecSrvr connection setup',
    45025: u'SecSrvr AuthEngine',
    45026: u'SecSrvr authinternal mech',
    45027: u'Calife',
    45028: u'sudo(1)',
    45029: u'audit crash recovery',
    45030: u'SecSrvr AuthMechanism',
    45031: u'Security Assessment'
}



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the OLE Compound File (OLECF) event formatters."""

import unittest

from plaso.formatters import olecf

from tests.formatters import test_lib


class OLECFItemFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the OLECF item event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = olecf.OLECFItemFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = olecf.OLECFItemFormatter()

    expected_attribute_names = [u'name']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class OLECFDestListEntryFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the DestList stream event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = olecf.OLECFDestListEntryFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = olecf.OLECFDestListEntryFormatter()

    expected_attribute_names = [
        u'entry_number',
        u'pin_status',
        u'hostname',
        u'path',
        u'droid_volume_identifier',
        u'droid_file_identifier',
        u'birth_droid_volume_identifier',
        u'birth_droid_file_identifier']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class OLECFDocumentSummaryInfoFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Document Summary Info property set stream event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = olecf.OLECFDocumentSummaryInfoFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = olecf.OLECFDocumentSummaryInfoFormatter()

    expected_attribute_names = [
        u'number_of_bytes',
        u'number_of_lines',
        u'number_of_paragraphs',
        u'number_of_slides',
        u'number_of_notes',
        u'number_of_hidden_slides',
        u'number_of_clips',
        u'company',
        u'manager',
        u'shared_document',
        u'application_version',
        u'content_type',
        u'content_status',
        u'language',
        u'document_version']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class OLECFSummaryInfoFormatter(test_lib.EventFormatterTestCase):
  """Tests for the Summary Info property set stream event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = olecf.OLECFSummaryInfoFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = olecf.OLECFSummaryInfoFormatter()

    expected_attribute_names = [
        u'title',
        u'subject',
        u'author',
        u'keywords',
        u'comments',
        u'template',
        u'revision_number',
        u'last_saved_by',
        u'total_edit_time',
        u'number_of_pages',
        u'number_of_words',
        u'number_of_characters',
        u'application',
        u'security']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the multi-processing analysis process."""

import unittest

from plaso.analysis import interface as analysis_interface
from plaso.containers import sessions
from plaso.multi_processing import analysis_process
from plaso.multi_processing import multi_process_queue

from tests.multi_processing import test_lib


class TestAnalysisPlugin(analysis_interface.AnalysisPlugin):
  """Analysis plugin for testing."""

  NAME = 'test_plugin'

  def CompileReport(self, unused_mediator):
    """Compiles a report of the analysis.

    After the plugin has received every copy of an event to
    analyze this function will be called so that the report
    can be assembled.

    Args:
      mediator (AnalysisMediator): mediates interactions between analysis
          plugins and other components, such as storage and dfvfs.

    Returns:
      AnalysisReport: analysis report.
    """
    return

  def ExamineEvent(self, unused_mediator, unused_event, **unused_kwargs):
    """Analyzes an event.

    Args:
      mediator (AnalysisMediator): mediates interactions between analysis
          plugins and other components, such as storage and dfvfs.
      event (EventObject): event.
    """
    return


class AnalysisProcessTest(test_lib.MultiProcessingTestCase):
  """Tests the multi-processing analysis process."""

  # pylint: disable=protected-access

  def testInitialization(self):
    """Tests the initialization."""
    test_process = analysis_process.AnalysisProcess(
        None, None, None, None, name=u'TestAnalysis')
    self.assertIsNotNone(test_process)

  def testGetStatus(self):
    """Tests the _GetStatus function."""
    test_process = analysis_process.AnalysisProcess(
        None, None, None, None, name=u'TestAnalysis')
    status_attributes = test_process._GetStatus()

    self.assertIsNotNone(status_attributes)
    self.assertEqual(status_attributes[u'identifier'], u'TestAnalysis')
    self.assertIsNone(status_attributes['number_of_produced_reports'])

    # TODO: add test with analysis mediator.

  def testMain(self):
    """Tests the _Main function."""
    event_queue = multi_process_queue.MultiProcessingQueue(timeout=1)

    session = sessions.Session()
    storage_writer = self._CreateStorageWriter(session)
    analysis_plugin = TestAnalysisPlugin()

    test_process = analysis_process.AnalysisProcess(
        event_queue, storage_writer, None, analysis_plugin,
        name=u'TestAnalysis')
    test_process._abort = True
    test_process._FOREMAN_STATUS_WAIT = 1
    test_process._pid = 0

    test_process._Main()

  # TODO: add test for _ProcessEvent.

  def testSignalAbort(self):
    """Tests the SignalAbort function."""
    test_process = analysis_process.AnalysisProcess(
        None, None, None, None, name=u'TestAnalysis')
    test_process.SignalAbort()


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Mac OS X appfirewall.log file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class MacAppFirewallLogFormatter(interface.ConditionalEventFormatter):
  """Formatter for Mac OS X appfirewall.log file event."""

  DATA_TYPE = u'mac:appfirewall:line'

  FORMAT_STRING_PIECES = [
      u'Computer: {computer_name}',
      u'Agent: {agent}',
      u'Status: {status}',
      u'Process name: {process_name}',
      u'Log: {action}']

  FORMAT_STRING_SHORT_PIECES = [
      u'Process name: {process_name}',
      u'Status: {status}']

  SOURCE_LONG = u'Mac AppFirewall Log'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(MacAppFirewallLogFormatter)



# -*- coding: utf-8 -*-
"""The Android WebView database event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class AndroidWebViewCookieEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for Android WebView Cookie event data."""

  DATA_TYPE = u'webview:cookie'

  FORMAT_STRING_PIECES = [
      u'Domain: {domain}',
      u'Path: {path}',
      u'Cookie name: {name}',
      u'Value: {value}',
      u'Secure: {secure}',]

  FORMAT_STRING_SHORT_PIECES = [
      u'{domain}',
      u'{name}',
      u'{value}']

  SOURCE_LONG = u'Android WebView'
  SOURCE_SHORT = u'WebView'


manager.FormattersManager.RegisterFormatter(AndroidWebViewCookieEventFormatter)



# -*- coding: utf-8 -*-
"""Parser for Google Chrome and Chromium Cache files."""

import os

import construct

from dfdatetime import webkit_time as dfdatetime_webkit_time
from dfvfs.resolver import resolver as path_spec_resolver
from dfvfs.path import factory as path_spec_factory

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.parsers import interface
from plaso.parsers import manager


class CacheAddress(object):
  """Chrome cache address.

  Attributes:
    block_number (int): block data file number.
    block_offset (int): offset within the block data file.
    block_size (int): block size.
    filename (str): name of the block data file.
    value (int): cache address.
  """
  FILE_TYPE_SEPARATE = 0
  FILE_TYPE_BLOCK_RANKINGS = 1
  FILE_TYPE_BLOCK_256 = 2
  FILE_TYPE_BLOCK_1024 = 3
  FILE_TYPE_BLOCK_4096 = 4

  _BLOCK_DATA_FILE_TYPES = [
      FILE_TYPE_BLOCK_RANKINGS,
      FILE_TYPE_BLOCK_256,
      FILE_TYPE_BLOCK_1024,
      FILE_TYPE_BLOCK_4096]

  _FILE_TYPE_BLOCK_SIZES = [0, 36, 256, 1024, 4096]

  def __init__(self, cache_address):
    """Initializes a cache address.

    Args:
      cache_address (int): cache address.
    """
    super(CacheAddress, self).__init__()
    self.block_number = None
    self.block_offset = None
    self.block_size = None
    self.filename = None
    self.value = cache_address

    if cache_address & 0x80000000:
      self.is_initialized = u'True'
    else:
      self.is_initialized = u'False'

    self.file_type = (cache_address & 0x70000000) >> 28
    if not cache_address == 0x00000000:
      if self.file_type == self.FILE_TYPE_SEPARATE:
        file_selector = cache_address & 0x0fffffff
        self.filename = u'f_{0:06x}'.format(file_selector)

      elif self.file_type in self._BLOCK_DATA_FILE_TYPES:
        file_selector = (cache_address & 0x00ff0000) >> 16
        self.filename = u'data_{0:d}'.format(file_selector)

        file_block_size = self._FILE_TYPE_BLOCK_SIZES[self.file_type]
        self.block_number = cache_address & 0x0000ffff
        self.block_size = (cache_address & 0x03000000) >> 24
        self.block_size *= file_block_size
        self.block_offset = 8192 + (self.block_number * file_block_size)


class CacheEntry(object):
  """Chrome cache entry.

  Attributes:
    creation_time (int): creation time, in number of micro seconds since
        January 1, 1970, 00:00:00 UTC.
    hash (int): super fast hash of the key.
    key (bytes): key.
    next (int): cache address of the next cache entry.
    rankings_node (int): cache address of the rankings node.
  """

  def __init__(self):
    """Initializes a cache entry."""
    super(CacheEntry, self).__init__()
    self.creation_time = None
    self.hash = None
    self.key = None
    self.next = None
    self.rankings_node = None


class IndexFile(object):
  """Chrome cache index file."""

  SIGNATURE = 0xc103cac3

  _FILE_HEADER = construct.Struct(
      u'chrome_cache_index_file_header',
      construct.ULInt32(u'signature'),
      construct.ULInt16(u'minor_version'),
      construct.ULInt16(u'major_version'),
      construct.ULInt32(u'number_of_entries'),
      construct.ULInt32(u'stored_data_size'),
      construct.ULInt32(u'last_created_file_number'),
      construct.ULInt32(u'unknown1'),
      construct.ULInt32(u'unknown2'),
      construct.ULInt32(u'table_size'),
      construct.ULInt32(u'unknown3'),
      construct.ULInt32(u'unknown4'),
      construct.ULInt64(u'creation_time'),
      construct.Padding(208))

  def __init__(self):
    """Initializes an index file."""
    super(IndexFile, self).__init__()
    self._file_object = None
    self.creation_time = None
    self.version = None
    self.index_table = []

  def _ReadFileHeader(self):
    """Reads the file header.

    Raises:
      IOError: if the file header cannot be read.
    """
    self._file_object.seek(0, os.SEEK_SET)

    try:
      file_header = self._FILE_HEADER.parse_stream(self._file_object)
    except construct.FieldError as exception:
      raise IOError(u'Unable to parse file header with error: {0:s}'.format(
          exception))

    signature = file_header.get(u'signature')

    if signature != self.SIGNATURE:
      raise IOError(u'Unsupported index file signature')

    self.version = u'{0:d}.{1:d}'.format(
        file_header.get(u'major_version'),
        file_header.get(u'minor_version'))

    if self.version not in [u'2.0', u'2.1']:
      raise IOError(u'Unsupported index file version: {0:s}'.format(
          self.version))

    self.creation_time = file_header.get(u'creation_time')

  def _ReadIndexTable(self):
    """Reads the index table."""
    cache_address_data = self._file_object.read(4)

    while len(cache_address_data) == 4:
      value = construct.ULInt32(u'cache_address').parse(cache_address_data)

      if value:
        cache_address = CacheAddress(value)
        self.index_table.append(cache_address)

      cache_address_data = self._file_object.read(4)

  def Close(self):
    """Closes the index file."""
    if self._file_object:
      self._file_object.close()
      self._file_object = None

  def Open(self, file_object):
    """Opens the index file.

    Args:
      file_object (file): file-like object.
    """
    self._file_object = file_object
    self._ReadFileHeader()
    # Skip over the LRU data, which is 112 bytes in size.
    self._file_object.seek(112, os.SEEK_CUR)
    self._ReadIndexTable()


class DataBlockFile(object):
  """Chrome data block file."""

  SIGNATURE = 0xc104cac3

  _FILE_HEADER = construct.Struct(
      u'chrome_cache_data_file_header',
      construct.ULInt32(u'signature'),
      construct.ULInt16(u'minor_version'),
      construct.ULInt16(u'major_version'),
      construct.ULInt16(u'file_number'),
      construct.ULInt16(u'next_file_number'),
      construct.ULInt32(u'block_size'),
      construct.ULInt32(u'number_of_entries'),
      construct.ULInt32(u'maximum_number_of_entries'),
      construct.Array(4, construct.ULInt32(u'emtpy')),
      construct.Array(4, construct.ULInt32(u'hints')),
      construct.ULInt32(u'updating'),
      construct.Array(5, construct.ULInt32(u'user')))

  _CACHE_ENTRY = construct.Struct(
      u'chrome_cache_entry',
      construct.ULInt32(u'hash'),
      construct.ULInt32(u'next_address'),
      construct.ULInt32(u'rankings_node_address'),
      construct.ULInt32(u'reuse_count'),
      construct.ULInt32(u'refetch_count'),
      construct.ULInt32(u'state'),
      construct.ULInt64(u'creation_time'),
      construct.ULInt32(u'key_size'),
      construct.ULInt32(u'long_key_address'),
      construct.Array(4, construct.ULInt32(u'data_stream_sizes')),
      construct.Array(4, construct.ULInt32(u'data_stream_addresses')),
      construct.ULInt32(u'flags'),
      construct.Padding(16),
      construct.ULInt32(u'self_hash'),
      construct.Array(160, construct.UBInt8(u'key')))

  def __init__(self):
    """Initializes a data block file."""
    super(DataBlockFile, self).__init__()
    self._file_object = None
    self.creation_time = None
    self.block_size = None
    self.number_of_entries = None
    self.version = None

  def _ReadFileHeader(self):
    """Reads the file header.

    Raises:
      IOError: if the file header cannot be read.
    """
    self._file_object.seek(0, os.SEEK_SET)

    try:
      file_header = self._FILE_HEADER.parse_stream(self._file_object)
    except construct.FieldError as exception:
      raise IOError(u'Unable to parse file header with error: {0:s}'.format(
          exception))

    signature = file_header.get(u'signature')

    if signature != self.SIGNATURE:
      raise IOError(u'Unsupported data block file signature')

    self.version = u'{0:d}.{1:d}'.format(
        file_header.get(u'major_version'),
        file_header.get(u'minor_version'))

    if self.version not in [u'2.0', u'2.1']:
      raise IOError(u'Unsupported data block file version: {0:s}'.format(
          self.version))

    self.block_size = file_header.get(u'block_size')
    self.number_of_entries = file_header.get(u'number_of_entries')

  def ReadCacheEntry(self, block_offset):
    """Reads a cache entry.

    Args:
      block_offset (int): block offset of the cache entry.

    Returns:
      CacheEntry: cache entry.
    """
    self._file_object.seek(block_offset, os.SEEK_SET)

    try:
      cache_entry_struct = self._CACHE_ENTRY.parse_stream(self._file_object)
    except construct.FieldError as exception:
      raise IOError(u'Unable to parse cache entry with error: {0:s}'.format(
          exception))

    cache_entry = CacheEntry()

    cache_entry.hash = cache_entry_struct.get(u'hash')

    cache_entry.next = CacheAddress(cache_entry_struct.get(u'next_address'))
    cache_entry.rankings_node = CacheAddress(cache_entry_struct.get(
        u'rankings_node_address'))

    cache_entry.creation_time = cache_entry_struct.get(u'creation_time')

    byte_array = cache_entry_struct.get(u'key')
    byte_string = b''.join(map(chr, byte_array))
    cache_entry.key, _, _ = byte_string.partition(b'\x00')

    return cache_entry

  def Close(self):
    """Closes the data block file."""
    if self._file_object:
      self._file_object.close()
      self._file_object = None

  def Open(self, file_object):
    """Opens the data block file.

    Args:
      file_object (file): file-like object.
    """
    self._file_object = file_object
    self._ReadFileHeader()


class ChromeCacheEntryEventData(events.EventData):
  """Chrome Cache event data.

  Attributes:
    original_url (str): original URL.
  """

  DATA_TYPE = u'chrome:cache:entry'

  def __init__(self):
    """Initializes event data."""
    super(ChromeCacheEntryEventData, self).__init__(data_type=self.DATA_TYPE)
    self.original_url = None


class ChromeCacheParser(interface.FileEntryParser):
  """Parses Chrome Cache files."""

  NAME = u'chrome_cache'
  DESCRIPTION = u'Parser for Chrome Cache files.'

  def _ParseCacheEntries(self, parser_mediator, index_file, data_block_files):
    """Parses Chrome Cache file entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      index_file (IndexFile): Chrome cache index file.
      data_block_files (dict[str: DataBlockFile]): data block files lookup table
          which contains data block files.
    """
    # Parse the cache entries in the data block files.
    for cache_address in index_file.index_table:
      cache_address_chain_length = 0
      while cache_address.value != 0x00000000:
        if cache_address_chain_length >= 64:
          parser_mediator.ProduceExtractionError(
              u'Maximum allowed cache address chain length reached.')
          break

        data_file = data_block_files.get(cache_address.filename, None)
        if not data_file:
          message = u'Cache address: 0x{0:08x} missing data file.'.format(
              cache_address.value)
          parser_mediator.ProduceExtractionError(message)
          break

        try:
          cache_entry = data_file.ReadCacheEntry(cache_address.block_offset)
        except (IOError, UnicodeDecodeError) as exception:
          parser_mediator.ProduceExtractionError(
              u'Unable to parse cache entry with error: {0:s}'.format(
                  exception))
          break

        try:
          original_url = cache_entry.key.decode(u'ascii')
        except UnicodeDecodeError:
          original_url = cache_entry.key.decode(u'ascii', errors=u'replace')
          parser_mediator.ProduceExtractionError((
              u'unable to decode cache entry key at cache address: '
              u'0x{0:08x}. Characters that cannot be decoded will be '
              u'replaced with "?" or "\\ufffd".').format(cache_address.value))

        event_data = ChromeCacheEntryEventData()
        event_data.original_url = original_url

        date_time = dfdatetime_webkit_time.WebKitTime(
            timestamp=cache_entry.creation_time)
        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_LAST_VISITED)
        parser_mediator.ProduceEventWithEventData(event, event_data)

        cache_address = cache_entry.next
        cache_address_chain_length += 1

  def ParseFileEntry(self, parser_mediator, file_entry, **kwargs):
    """Parses Chrome Cache files.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_entry (dfvfs.FileEntry): file entry.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    index_file = IndexFile()
    file_object = file_entry.GetFileObject()
    try:
      index_file.Open(file_object)
    except IOError as exception:
      file_object.close()

      display_name = parser_mediator.GetDisplayName()
      raise errors.UnableToParseFile(
          u'[{0:s}] unable to parse index file {1:s} with error: {2:s}'.format(
              self.NAME, display_name, exception))

    try:
      file_system = file_entry.GetFileSystem()
      self.ParseIndexFile(
          parser_mediator, file_system, file_entry, index_file, **kwargs)
    finally:
      index_file.Close()

  def ParseIndexFile(
      self, parser_mediator, file_system, file_entry, index_file, **kwargs):
    """Parses a Chrome Cache index file object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_system (dfvfs.FileSystem): file system.
      file_entry (dfvfs.FileEntry): file entry.
      index_file (IndexFile): Chrome cache index file.
    """
    # Build a lookup table for the data block files.
    path_segments = file_system.SplitPath(file_entry.path_spec.location)

    data_block_files = {}
    for cache_address in index_file.index_table:
      if cache_address.filename not in data_block_files:
        # Remove the previous filename from the path segments list and
        # add one of the data block file.
        path_segments.pop()
        path_segments.append(cache_address.filename)

        # We need to pass only used arguments to the path specification
        # factory otherwise it will raise.
        kwargs = {}
        if file_entry.path_spec.parent:
          kwargs[u'parent'] = file_entry.path_spec.parent
        kwargs[u'location'] = file_system.JoinPath(path_segments)

        data_block_file_path_spec = path_spec_factory.Factory.NewPathSpec(
            file_entry.path_spec.TYPE_INDICATOR, **kwargs)

        try:
          data_block_file_entry = path_spec_resolver.Resolver.OpenFileEntry(
              data_block_file_path_spec)
        except RuntimeError as exception:
          message = (
              u'Unable to open data block file: {0:s} with error: '
              u'{1:s}'.format(kwargs[u'location'], exception))
          parser_mediator.ProduceExtractionError(message)
          data_block_file_entry = None

        if not data_block_file_entry:
          message = u'Missing data block file: {0:s}'.format(
              cache_address.filename)
          parser_mediator.ProduceExtractionError(message)
          data_block_file = None

        else:
          data_block_file_object = data_block_file_entry.GetFileObject()
          data_block_file = DataBlockFile()

          try:
            data_block_file.Open(data_block_file_object)
          except IOError as exception:
            message = (
                u'Unable to open data block file: {0:s} with error: '
                u'{1:s}').format(cache_address.filename, exception)
            parser_mediator.ProduceExtractionError(message)
            data_block_file = None

        data_block_files[cache_address.filename] = data_block_file

    try:
      self._ParseCacheEntries(parser_mediator, index_file, data_block_files)
    finally:
      for data_block_file in iter(data_block_files.values()):
        if data_block_file:
          data_block_file.Close()


manager.ParsersManager.RegisterParser(ChromeCacheParser)



# -*- coding: utf-8 -*-
"""This file contains the Property List (Plist) Parser.

Plaso's engine calls PlistParser when it encounters Plist files to be processed.
"""

import binascii
import logging

from binplist import binplist

from plaso.lib import errors
from plaso.lib import py2to3
from plaso.parsers import interface
from plaso.parsers import manager


class PlistParser(interface.FileObjectParser):
  """Parses binary and text plist plist files.

  The Plaso engine calls parsers by their Parse() method. This parser's
  Parse() has GetTopLevel() which deserializes plist files using the binplist
  library and calls plugins (PlistPlugin) registered through the
  interface by their Process() to produce event objects.

  Plugins are how this parser understands the content inside a plist file,
  each plugin holds logic specific to a particular plist file. See the
  interface and plist_plugins/ directory for examples of how plist plugins are
  implemented.
  """

  NAME = u'plist'
  DESCRIPTION = u'Parser for binary and text plist files.'

  _plugin_classes = {}

  def GetTopLevel(self, file_object, file_name=u''):
    """Returns the deserialized content of a plist as a dictionary object.

    Args:
      file_object (dfvfs.FileIO): a file-like object to parse.
      file_name (str): name of the file-like object.

    Returns:
      dict[str, object]: contents of the plist.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    # Note that binplist.readPlist does not seek to offset 0.
    try:
      top_level_object = binplist.readPlist(file_object)

    except binplist.FormatError as exception:
      if not isinstance(exception, py2to3.BYTES_TYPE):
        error_string = str(exception).decode(u'utf8', errors=u'replace')
      else:
        error_string = exception

      raise errors.UnableToParseFile(
          u'File is not a plist file: {0:s}'.format(error_string))

    except (
        AttributeError, LookupError, ValueError, binascii.Error) as exception:
      raise errors.UnableToParseFile(
          u'Unable to parse XML file, reason: {0:s}'.format(exception))

    except OverflowError as exception:
      raise errors.UnableToParseFile(
          u'Unable to parse: {0:s} with error: {1:s}'.format(
              file_name, exception))

    if not top_level_object:
      raise errors.UnableToParseFile(
          u'File is not a plist: missing top level object')

    # Since we are using readPlist from binplist now instead of manually
    # opening  the binary plist file we loose this option. Keep it commented
    # out for now but this needs to be tested a bit more.
    # TODO: Re-evaluate if we can delete this or still require it.
    #if bpl.is_corrupt:
    #  logging.warning(
    #      u'Corruption detected in binary plist: {0:s}'.format(file_name))

    return top_level_object

  def ParseFileObject(self, parser_mediator, file_object, **unused_kwargs):
    """Parses a plist file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    filename = parser_mediator.GetFilename()
    file_size = file_object.get_size()

    if file_size <= 0:
      raise errors.UnableToParseFile(
          u'File size: {0:d} bytes is less equal 0.'.format(file_size))

    # 50MB is 10x larger than any plist seen to date.
    if file_size > 50000000:
      raise errors.UnableToParseFile(
          u'File size: {0:d} bytes is larger than 50 MB.'.format(file_size))

    top_level_object = None
    try:
      top_level_object = self.GetTopLevel(file_object, filename)
    except errors.UnableToParseFile:
      raise

    if not top_level_object:
      raise errors.UnableToParseFile(
          u'Unable to parse: {0:s} skipping.'.format(filename))

    # TODO: add a parser filter.
    matching_plugin = None
    for plugin in self._plugins:
      try:
        plugin.UpdateChainAndProcess(
            parser_mediator, plist_name=filename, top_level=top_level_object)
        matching_plugin = plugin

      except errors.WrongPlistPlugin as exception:
        logging.debug(u'Wrong plugin: {0:s} for: {1:s}'.format(
            exception.args[0], exception.args[1]))

    if not matching_plugin and self._default_plugin:
      self._default_plugin.UpdateChainAndProcess(
          parser_mediator, plist_name=filename, top_level=top_level_object)


manager.ParsersManager.RegisterParser(PlistParser)
